"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
require("reflect-metadata");
const Main_elm_1 = tslib_1.__importDefault(require("../elm/Stellerator/Main.elm"));
require("../theme/dos.scss");
const rangetouch_1 = require("../common/rangetouch");
const CartridgeInfo_1 = tslib_1.__importDefault(require("../../machine/stella/cartridge/CartridgeInfo"));
const MediaApi_1 = tslib_1.__importDefault(require("./service/MediaApi"));
const ScrollIntoView_1 = tslib_1.__importDefault(require("./service/ScrollIntoView"));
const AddCartridge_1 = tslib_1.__importDefault(require("./service/AddCartridge"));
const inversify_1 = require("inversify");
const Storage_1 = tslib_1.__importDefault(require("./service/Storage"));
const TrackCartridges_1 = tslib_1.__importDefault(require("./service/TrackCartridges"));
const TrackSettings_1 = tslib_1.__importDefault(require("./service/TrackSettings"));
const Emulation_1 = tslib_1.__importDefault(require("./service/Emulation"));
const TouchIO_1 = tslib_1.__importDefault(require("../../web/stella/driver/TouchIO"));
const Gamepad_1 = tslib_1.__importDefault(require("../../web/driver/Gamepad"));
const Capabilities_1 = require("../../web/driver/video/Capabilities");
const VERSION_STORAGE_KEY = process.env.DEVELOPMENT ? 'stellerator-ng-version-dev' : 'stellerator-ng-version';
if (navigator.serviceWorker && !process.env.DEVELOPMENT) {
    navigator.serviceWorker.register('./service-worker.js', { scope: './' });
}
const defaultSettings = (badGpu) => ({
    cpuEmulation: "cycle",
    volume: 80,
    audioEmulation: "pcm",
    gammaCorrection: 1.0,
    tvEmulation: badGpu ? "none" : "composite",
    scaling: badGpu ? "bilinear" : "qis",
    phosphorLevel: 50,
    scanlineIntensity: 20,
    touchControls: undefined,
    leftHanded: false,
    virtualJoystickSensitivity: 10,
    uiMode: undefined,
    uiSize: 100
});
function main() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        rangetouch_1.initialize();
        const cartridgeTypes = CartridgeInfo_1.default.getAllTypes().map(cartridgeType => ({
            key: cartridgeType,
            description: CartridgeInfo_1.default.describeCartridgeType(cartridgeType)
        }));
        const container = new inversify_1.Container({ autoBindInjectable: true, defaultScope: 'Singleton' });
        const storage = container.get(Storage_1.default);
        const capabilities = Capabilities_1.detect();
        const badGpu = !(capabilities &&
            (capabilities.floatTextures || capabilities.halfFloatTextures) &&
            capabilities.highpInFsh);
        storage.setDefaults(defaultSettings(badGpu));
        const [cartridges, settings] = yield Promise.all([storage.getAllCartridges(), storage.getSettings()]);
        const version = process.env.VERSION;
        const oldVersion = localStorage.getItem(VERSION_STORAGE_KEY);
        const wasUpdated = !!oldVersion && oldVersion !== version;
        localStorage.setItem(VERSION_STORAGE_KEY, version);
        const { ports } = Main_elm_1.default.Stellerator.Main.init({
            flags: {
                cartridges,
                cartridgeTypes,
                settings,
                defaultSettings: defaultSettings(badGpu),
                touchSupport: TouchIO_1.default.isSupported(),
                version,
                wasUpdated,
                gamepadCount: Gamepad_1.default.probeGamepadCount(),
                badGpu
            }
        });
        container.get(MediaApi_1.default).init(ports);
        container.get(ScrollIntoView_1.default).init(ports);
        container.get(AddCartridge_1.default).init(ports);
        container.get(TrackCartridges_1.default).init(ports);
        container.get(TrackSettings_1.default).init(ports);
        container.get(Emulation_1.default).init(ports);
        ports.scrollToTop_.subscribe(() => window.scrollTo(0, 0));
        ports.blurCurrentElement_.subscribe(() => document.activeElement.blur());
    });
}
window.addEventListener('load', main);
//# sourceMappingURL=index.js.map