"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const inversify_1 = require("inversify");
let ScrollIntoView = class ScrollIntoView {
    constructor() {
        this._scrollIntoView = ([position, id]) => {
            const node = document.getElementById(id);
            if (node) {
                node.scrollIntoView({ block: position });
            }
            else {
                new Task(position, id).start();
            }
        };
    }
    init(ports) {
        ports.scrollIntoView_.subscribe(this._scrollIntoView);
    }
};
ScrollIntoView = tslib_1.__decorate([
    inversify_1.injectable()
], ScrollIntoView);
class Task {
    constructor(_position, _id, _timeout = 500) {
        this._position = _position;
        this._id = _id;
        this._timeout = _timeout;
        this._onMutation = (mutations) => {
            if (!mutations.some(m => m.addedNodes)) {
                return;
            }
            const node = document.getElementById(this._id);
            if (node) {
                this._observer.disconnect();
                clearTimeout(this._timeoutHandle);
                node.scrollIntoView({ block: this._position });
            }
        };
        this._observer = new MutationObserver(this._onMutation);
        this._timeoutHandle = null;
    }
    start() {
        setTimeout(() => this._observer.disconnect(), this._timeout);
        this._observer.observe(document.body, {
            attributes: false,
            characterData: false,
            childList: true,
            subtree: true
        });
    }
}
exports.default = ScrollIntoView;
//# sourceMappingURL=ScrollIntoView.js.map