"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const inversify_1 = require("inversify");
const Storage_1 = tslib_1.__importDefault(require("./Storage"));
const Emulation_1 = tslib_1.__importDefault(require("./Emulation"));
const async_mutex_1 = require("async-mutex");
const file_saver_1 = tslib_1.__importDefault(require("file-saver"));
const Throttle_1 = tslib_1.__importDefault(require("../../../tools/Throttle"));
let TrackCartridges = class TrackCartridges {
    constructor(_storage, _emulation) {
        this._storage = _storage;
        this._emulation = _emulation;
        this._onCartridgeDeleted = (hash) => this._storage.deleteCartridge(hash);
        this._onDeleteAllCartridges = () => this._storage.deleteAllCartridges();
        this._onSaveCartridge = (hash) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [cartridge, image] = yield Promise.all([
                this._storage.getCartridge(hash),
                this._storage.getCartridgeImage(hash)
            ]);
            if (!cartridge || !image) {
                return;
            }
            file_saver_1.default.saveAs(new Blob([image], { type: 'application/octet-stream' }), cartridge.name + '.bin', {
                autoBom: false
            });
        });
        this._updateCartridgeThrottle = new Throttle_1.default(250, cartridge => this._mutex.runExclusive(() => this._updateCartridge(cartridge)));
        this._mutex = new async_mutex_1.Mutex();
    }
    init(ports) {
        ports.updateCartridge_.subscribe(this._updateCartridgeThrottle.run.bind(this._updateCartridgeThrottle));
        ports.deleteCartridge_.subscribe(this._onCartridgeDeleted);
        ports.deleteAllCartridges_.subscribe(this._onDeleteAllCartridges);
        ports.saveCartridge_.subscribe(this._onSaveCartridge);
    }
    _updateCartridge(cartridge) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this._storage.updateCartridge(cartridge);
            yield this._emulation.updateCartridge(cartridge);
        });
    }
};
TrackCartridges = tslib_1.__decorate([
    inversify_1.injectable(),
    tslib_1.__param(0, inversify_1.inject(Storage_1.default)), tslib_1.__param(1, inversify_1.inject(Emulation_1.default)),
    tslib_1.__metadata("design:paramtypes", [Storage_1.default, Emulation_1.default])
], TrackCartridges);
exports.default = TrackCartridges;
//# sourceMappingURL=TrackCartridges.js.map