"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const inversify_1 = require("inversify");
const Storage_1 = tslib_1.__importDefault(require("./Storage"));
const Emulation_1 = tslib_1.__importDefault(require("./Emulation"));
const async_mutex_1 = require("async-mutex");
const Throttle_1 = tslib_1.__importDefault(require("../../../tools/Throttle"));
let TrackSettings = class TrackSettings {
    constructor(_storage, _emulation) {
        this._storage = _storage;
        this._emulation = _emulation;
        this._throttle = new Throttle_1.default(250, this._applyUpdate.bind(this));
        this._mutex = new async_mutex_1.Mutex();
    }
    init(ports) {
        ports.updateSettings_.subscribe(this._throttle.run.bind(this._throttle));
    }
    _applyUpdate(settings) {
        this._mutex.runExclusive(() => tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this._storage.saveSettings(settings);
            yield this._emulation.updateSettings(settings);
        }));
    }
};
TrackSettings = tslib_1.__decorate([
    inversify_1.injectable(),
    tslib_1.__param(0, inversify_1.inject(Storage_1.default)), tslib_1.__param(1, inversify_1.inject(Emulation_1.default)),
    tslib_1.__metadata("design:paramtypes", [Storage_1.default, Emulation_1.default])
], TrackSettings);
exports.default = TrackSettings;
//# sourceMappingURL=TrackSettings.js.map