"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const Instruction_1 = tslib_1.__importDefault(require("./Instruction"));
const hex = tslib_1.__importStar(require("../../tools/hex"));
class Disassembler {
    constructor(_bus) {
        this._bus = _bus;
    }
    setBus(bus) {
        this._bus = bus;
        return this;
    }
    getBus() {
        return this._bus;
    }
    disassembleAt(address) {
        const instruction = Instruction_1.default.opcodes[this._peek(address)], operation = Instruction_1.default.OperationMap[instruction.operation].toUpperCase();
        const read8 = (a = address + 1) => hex.encode(this._peek(a), 2);
        const read16 = (a = address + 1) => hex.encode(this._peek(a) + (this._peek(a + 1) << 8), 4);
        const decodeSint8 = (value) => (value & 0x80 ? -(~(value - 1) & 0xff) : value);
        switch (instruction.effectiveAddressingMode) {
            case 0:
                return operation;
            case 1:
                return operation + ' #' + read8();
            case 2:
                return operation + ' ' + read8();
            case 3:
                return operation + ' ' + read16();
            case 4:
                return operation + ' (' + read16() + ')';
            case 5:
                const distance = decodeSint8(this._peek(address + 1));
                return (operation +
                    ' ' +
                    hex.encode(distance, 2) +
                    ' ; -> ' +
                    hex.encode((0x10002 + address + distance) % 0x10000, 4));
            case 6:
                return operation + ' ' + read8() + ',X';
            case 7:
                return operation + ' ' + read16() + ',X';
            case 8:
                return operation + ' (' + read8() + ',X)';
            case 9:
                return operation + ' ' + read8() + ',Y';
            case 10:
                return operation + ' ' + read16() + ',Y';
            case 11:
                return operation + ' (' + read8() + '),Y';
            default:
                return 'INVALID';
        }
    }
    _peek(address) {
        return this._bus.peek(address % 0x10000);
    }
}
exports.default = Disassembler;
//# sourceMappingURL=Disassembler.js.map