"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.opRla = exports.opRra = exports.opAtx = exports.opAac = exports.opIsc = exports.opLar = exports.opAax = exports.opSlo = exports.opArr = exports.opLax = exports.opDcp = exports.opAxs = exports.opAlr = exports.opTya = exports.opTxs = exports.opTxa = exports.opTsx = exports.opTay = exports.opTax = exports.opSty = exports.opStx = exports.opSta = exports.opSei = exports.opSed = exports.opSec = exports.opSbc = exports.opRts = exports.opRti = exports.opRorMem = exports.opRorAcc = exports.opRolMem = exports.opRolAcc = exports.opPla = exports.opPha = exports.opPlp = exports.opPhp = exports.opOra = exports.opNop = exports.opLsrMem = exports.opLsrAcc = exports.opLdy = exports.opLdx = exports.opLda = exports.opJsr = exports.opJmp = exports.opIny = exports.opInx = exports.opInc = exports.opDey = exports.opEor = exports.opDex = exports.opDec = exports.opCpy = exports.opCpx = exports.opCmp = exports.opClv = exports.opCli = exports.opCld = exports.opClc = exports.opBrk = exports.opBit = exports.opAslMem = exports.opAslAcc = exports.opAnd = exports.opAdc = void 0;
function restoreFlagsFromStack(state, bus) {
    state.s = (state.s + 0x01) & 0xff;
    state.flags = (bus.read(0x0100 + state.s) | 32) & ~16;
}
function setFlagsNZ(state, operand) {
    state.flags =
        (state.flags & ~(128 | 2)) |
            (operand & 0x80) |
            (operand ? 0 : 2);
}
function opAdc(state, bus, operand) {
    if (state.flags & 8) {
        const d0 = (operand & 0x0f) + (state.a & 0x0f) + (state.flags & 1), d1 = (operand >>> 4) + (state.a >>> 4) + (d0 > 9 ? 1 : 0);
        state.a = d0 % 10 | (d1 % 10 << 4);
        state.flags =
            (state.flags & ~(128 | 2 | 1)) |
                (state.a & 0x80) |
                (state.a ? 0 : 2) |
                (d1 > 9 ? 1 : 0);
    }
    else {
        const sum = state.a + operand + (state.flags & 1), result = sum & 0xff;
        state.flags =
            (state.flags &
                ~(128 | 2 | 1 | 64)) |
                (result & 0x80) |
                (result ? 0 : 2) |
                (sum >>> 8) |
                ((~(operand ^ state.a) & (result ^ operand) & 0x80) >>> 1);
        state.a = result;
    }
}
exports.opAdc = opAdc;
function opAnd(state, bus, operand) {
    state.a &= operand;
    setFlagsNZ(state, state.a);
}
exports.opAnd = opAnd;
function opAslAcc(state) {
    const old = state.a;
    state.a = (state.a << 1) & 0xff;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (state.a & 0x80) |
            (state.a ? 0 : 2) |
            (old >>> 7);
}
exports.opAslAcc = opAslAcc;
function opAslMem(state, bus, operand) {
    const old = bus.read(operand), value = (old << 1) & 0xff;
    bus.write(operand, value);
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (value & 0x80) |
            (value ? 0 : 2) |
            (old >>> 7);
}
exports.opAslMem = opAslMem;
function opBit(state, bus, operand) {
    state.flags =
        (state.flags & ~(128 | 64 | 2)) |
            (operand & (128 | 64)) |
            (operand & state.a ? 0 : 2);
}
exports.opBit = opBit;
function opBrk(state, bus) {
    const nextOpAddr = (state.p + 1) & 0xffff;
    let vector = 0xfffe;
    if (state.nmi) {
        vector = 0xfffa;
        state.nmi = false;
    }
    state.nmi = state.irq = false;
    bus.write(state.s + 0x0100, (nextOpAddr >>> 8) & 0xff);
    state.s = (state.s + 0xff) & 0xff;
    bus.write(state.s + 0x0100, nextOpAddr & 0xff);
    state.s = (state.s + 0xff) & 0xff;
    bus.write(state.s + 0x0100, state.flags | 16);
    state.s = (state.s + 0xff) & 0xff;
    state.flags |= 4;
    state.p = bus.readWord(vector);
}
exports.opBrk = opBrk;
function opClc(state) {
    state.flags &= ~1;
}
exports.opClc = opClc;
function opCld(state) {
    state.flags &= ~8;
}
exports.opCld = opCld;
function opCli(state) {
    state.flags &= ~4;
}
exports.opCli = opCli;
function opClv(state) {
    state.flags &= ~64;
}
exports.opClv = opClv;
function opCmp(state, bus, operand) {
    const diff = state.a + (~operand & 0xff) + 1;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (diff & 0x80) |
            (diff & 0xff ? 0 : 2) |
            (diff >>> 8);
}
exports.opCmp = opCmp;
function opCpx(state, bus, operand) {
    const diff = state.x + (~operand & 0xff) + 1;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (diff & 0x80) |
            (diff & 0xff ? 0 : 2) |
            (diff >>> 8);
}
exports.opCpx = opCpx;
function opCpy(state, bus, operand) {
    const diff = state.y + (~operand & 0xff) + 1;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (diff & 0x80) |
            (diff & 0xff ? 0 : 2) |
            (diff >>> 8);
}
exports.opCpy = opCpy;
function opDec(state, bus, operand) {
    const value = (bus.read(operand) + 0xff) & 0xff;
    bus.write(operand, value);
    setFlagsNZ(state, value);
}
exports.opDec = opDec;
function opDex(state) {
    state.x = (state.x + 0xff) & 0xff;
    setFlagsNZ(state, state.x);
}
exports.opDex = opDex;
function opEor(state, bus, operand) {
    state.a = state.a ^ operand;
    setFlagsNZ(state, state.a);
}
exports.opEor = opEor;
function opDey(state) {
    state.y = (state.y + 0xff) & 0xff;
    setFlagsNZ(state, state.y);
}
exports.opDey = opDey;
function opInc(state, bus, operand) {
    const value = (bus.read(operand) + 1) & 0xff;
    bus.write(operand, value);
    setFlagsNZ(state, value);
}
exports.opInc = opInc;
function opInx(state) {
    state.x = (state.x + 0x01) & 0xff;
    setFlagsNZ(state, state.x);
}
exports.opInx = opInx;
function opIny(state) {
    state.y = (state.y + 0x01) & 0xff;
    setFlagsNZ(state, state.y);
}
exports.opIny = opIny;
function opJmp(state, bus, operand) {
    state.p = operand;
}
exports.opJmp = opJmp;
function opJsr(state, bus, operand) {
    const returnPtr = (state.p + 1) & 0xffff, addrLo = bus.read(state.p);
    bus.read(0x0100 + state.s);
    bus.write(0x0100 + state.s, returnPtr >>> 8);
    state.s = (state.s + 0xff) & 0xff;
    bus.write(0x0100 + state.s, returnPtr & 0xff);
    state.s = (state.s + 0xff) & 0xff;
    state.p = addrLo | (bus.read((state.p + 1) & 0xffff) << 8);
}
exports.opJsr = opJsr;
function opLda(state, bus, operand, addressingMode) {
    state.a = addressingMode === 1 ? operand : bus.read(operand);
    setFlagsNZ(state, state.a);
}
exports.opLda = opLda;
function opLdx(state, bus, operand, addressingMode) {
    state.x = addressingMode === 1 ? operand : bus.read(operand);
    setFlagsNZ(state, state.x);
}
exports.opLdx = opLdx;
function opLdy(state, bus, operand, addressingMode) {
    state.y = addressingMode === 1 ? operand : bus.read(operand);
    setFlagsNZ(state, state.y);
}
exports.opLdy = opLdy;
function opLsrAcc(state) {
    const old = state.a;
    state.a = state.a >>> 1;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (state.a & 0x80) |
            (state.a ? 0 : 2) |
            (old & 1);
}
exports.opLsrAcc = opLsrAcc;
function opLsrMem(state, bus, operand) {
    const old = bus.read(operand), value = old >>> 1;
    bus.write(operand, value);
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (value & 0x80) |
            (value ? 0 : 2) |
            (old & 1);
}
exports.opLsrMem = opLsrMem;
function opNop() { }
exports.opNop = opNop;
function opOra(state, bus, operand) {
    state.a |= operand;
    setFlagsNZ(state, state.a);
}
exports.opOra = opOra;
function opPhp(state, bus) {
    bus.write(0x0100 + state.s, state.flags | 16);
    state.s = (state.s + 0xff) & 0xff;
}
exports.opPhp = opPhp;
function opPlp(state, bus) {
    restoreFlagsFromStack(state, bus);
}
exports.opPlp = opPlp;
function opPha(state, bus) {
    bus.write(0x0100 + state.s, state.a);
    state.s = (state.s + 0xff) & 0xff;
}
exports.opPha = opPha;
function opPla(state, bus) {
    state.s = (state.s + 0x01) & 0xff;
    state.a = bus.read(0x0100 + state.s);
    setFlagsNZ(state, state.a);
}
exports.opPla = opPla;
function opRolAcc(state) {
    const old = state.a;
    state.a = ((state.a << 1) & 0xff) | (state.flags & 1);
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (state.a & 0x80) |
            (state.a ? 0 : 2) |
            (old >>> 7);
}
exports.opRolAcc = opRolAcc;
function opRolMem(state, bus, operand) {
    const old = bus.read(operand), value = ((old << 1) & 0xff) | (state.flags & 1);
    bus.write(operand, value);
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (value & 0x80) |
            (value ? 0 : 2) |
            (old >>> 7);
}
exports.opRolMem = opRolMem;
function opRorAcc(state) {
    const old = state.a;
    state.a = (state.a >>> 1) | ((state.flags & 1) << 7);
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (state.a & 0x80) |
            (state.a ? 0 : 2) |
            (old & 1);
}
exports.opRorAcc = opRorAcc;
function opRorMem(state, bus, operand) {
    const old = bus.read(operand), value = (old >>> 1) | ((state.flags & 1) << 7);
    bus.write(operand, value);
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (value & 0x80) |
            (value ? 0 : 2) |
            (old & 1);
}
exports.opRorMem = opRorMem;
function opRti(state, bus) {
    let returnPtr;
    restoreFlagsFromStack(state, bus);
    state.s = (state.s + 1) & 0xff;
    returnPtr = bus.read(0x0100 + state.s);
    state.s = (state.s + 1) & 0xff;
    returnPtr |= bus.read(0x0100 + state.s) << 8;
    state.p = returnPtr;
}
exports.opRti = opRti;
function opRts(state, bus) {
    let returnPtr;
    bus.read(0x0100 + state.s);
    state.s = (state.s + 1) & 0xff;
    returnPtr = bus.read(0x0100 + state.s);
    state.s = (state.s + 1) & 0xff;
    returnPtr += bus.read(0x0100 + state.s) << 8;
    state.p = (returnPtr + 1) & 0xffff;
}
exports.opRts = opRts;
function opSbc(state, bus, operand) {
    if (state.flags & 8) {
        const d0 = (state.a & 0x0f) - (operand & 0x0f) - (~state.flags & 1), d1 = (state.a >>> 4) - (operand >>> 4) - (d0 < 0 ? 1 : 0);
        state.a = (d0 < 0 ? 10 + d0 : d0) | ((d1 < 0 ? 10 + d1 : d1) << 4);
        state.flags =
            (state.flags & ~(128 | 2 | 1)) |
                (state.a & 0x80) |
                (state.a ? 0 : 2) |
                (d1 < 0 ? 0 : 1);
    }
    else {
        operand = ~operand & 0xff;
        const sum = state.a + operand + (state.flags & 1), result = sum & 0xff;
        state.flags =
            (state.flags &
                ~(128 | 2 | 1 | 64)) |
                (result & 0x80) |
                (result ? 0 : 2) |
                (sum >>> 8) |
                ((~(operand ^ state.a) & (result ^ operand) & 0x80) >>> 1);
        state.a = result;
    }
}
exports.opSbc = opSbc;
function opSec(state) {
    state.flags |= 1;
}
exports.opSec = opSec;
function opSed(state) {
    state.flags |= 8;
}
exports.opSed = opSed;
function opSei(state) {
    state.flags |= 4;
}
exports.opSei = opSei;
function opSta(state, bus, operand) {
    bus.write(operand, state.a);
}
exports.opSta = opSta;
function opStx(state, bus, operand) {
    bus.write(operand, state.x);
}
exports.opStx = opStx;
function opSty(state, bus, operand) {
    bus.write(operand, state.y);
}
exports.opSty = opSty;
function opTax(state) {
    state.x = state.a;
    setFlagsNZ(state, state.a);
}
exports.opTax = opTax;
function opTay(state) {
    state.y = state.a;
    setFlagsNZ(state, state.a);
}
exports.opTay = opTay;
function opTsx(state) {
    state.x = state.s;
    setFlagsNZ(state, state.x);
}
exports.opTsx = opTsx;
function opTxa(state) {
    state.a = state.x;
    setFlagsNZ(state, state.a);
}
exports.opTxa = opTxa;
function opTxs(state) {
    state.s = state.x;
}
exports.opTxs = opTxs;
function opTya(state) {
    state.a = state.y;
    setFlagsNZ(state, state.a);
}
exports.opTya = opTya;
function opAlr(state, bus, operand) {
    const i = state.a & operand;
    state.a = i >>> 1;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (state.a & 0x80) |
            (state.a ? 0 : 2) |
            (i & 1);
}
exports.opAlr = opAlr;
function opAxs(state, bus, operand) {
    const value = (state.a & state.x) + (~operand & 0xff) + 1;
    state.x = value & 0xff;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (state.x & 0x80) |
            (state.x & 0xff ? 0 : 2) |
            (value >>> 8);
}
exports.opAxs = opAxs;
function opDcp(state, bus, operand) {
    const value = (bus.read(operand) + 0xff) & 0xff;
    bus.write(operand, value);
    const diff = state.a + (~value & 0xff) + 1;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (diff & 0x80) |
            (diff & 0xff ? 0 : 2) |
            (diff >>> 8);
}
exports.opDcp = opDcp;
function opLax(state, bus, operand) {
    state.a = operand;
    state.x = operand;
    setFlagsNZ(state, operand);
}
exports.opLax = opLax;
function opArr(state, bus, operand) {
    state.a = ((state.a & operand) >>> 1) | (state.flags & 1 ? 0x80 : 0);
    state.flags =
        (state.flags & ~(1 | 128 | 2 | 64)) |
            ((state.a & 0x40) >>> 6) |
            (state.a ? 0 : 2) |
            (state.a & 0x80) |
            ((state.a & 0x40) ^ ((state.a & 0x20) << 1));
}
exports.opArr = opArr;
function opSlo(state, bus, operand) {
    let value = bus.read(operand);
    state.flags = (state.flags & ~1) | (value >>> 7);
    value = (value << 1) & 0xff;
    bus.write(operand, value);
    state.a = state.a | value;
    setFlagsNZ(state, state.a);
}
exports.opSlo = opSlo;
function opAax(state, bus, operand) {
    const value = state.x & state.a;
    bus.write(operand, value);
    setFlagsNZ(state, value);
}
exports.opAax = opAax;
function opLar(state, bus, operand) {
    state.s = state.a = state.x = state.s & operand;
    setFlagsNZ(state, state.a);
}
exports.opLar = opLar;
function opIsc(state, bus, operand) {
    const value = (bus.read(operand) + 1) & 0xff;
    bus.write(operand, value);
    opSbc(state, bus, value);
}
exports.opIsc = opIsc;
function opAac(state, bus, operand) {
    state.a &= operand;
    setFlagsNZ(state, state.a);
    state.flags = (state.flags & ~1) | ((state.a & 0x80) >>> 7);
}
exports.opAac = opAac;
function opAtx(state, bus, operand) {
    state.a &= operand;
    state.x = state.a;
    setFlagsNZ(state, state.a);
}
exports.opAtx = opAtx;
function opRra(state, bus, operand) {
    const old = bus.read(operand), value = (old >>> 1) | ((state.flags & 1) << 7);
    bus.write(operand, value);
    state.flags = (state.flags & ~1) | (old & 1);
    opAdc(state, bus, value);
}
exports.opRra = opRra;
function opRla(state, bus, operand) {
    const old = bus.read(operand), value = ((old << 1) & 0xff) | (state.flags & 1);
    bus.write(operand, value);
    state.flags = (state.flags & ~1) | (old >>> 7);
    opAnd(state, bus, value);
}
exports.opRla = opRla;
//# sourceMappingURL=ops.js.map