"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Instruction_1 = tslib_1.__importDefault(require("../Instruction"));
const addressing_1 = require("./addressing");
const instruction_1 = require("./instruction");
const ops = tslib_1.__importStar(require("./ops"));
const indirect_1 = require("./addressing/indirect");
const vector_1 = require("./vector");
class Compiler {
    constructor(_state) {
        this._state = _state;
    }
    compile(op) {
        const instruction = Instruction_1.default.opcodes[op];
        switch (instruction.operation) {
            case 0:
                return this._createAddressing(instruction.addressingMode, ops.adc, {
                    deref: true
                });
            case 1:
                return this._createAddressing(instruction.addressingMode, (o, s) => ops.genUnary(o, s, (operand, state) => (state.a = state.a & operand)), {
                    deref: true
                });
            case 2:
                return instruction.addressingMode === 0
                    ? instruction_1.nullaryOneCycle(this._state, ops.aslImmediate)
                    : this._createAddressing(instruction.addressingMode, instruction_1.readModifyWrite(this._state, ops.aslRmw).reset, { writeOp: true });
            case 6:
                return this._createAddressing(instruction.addressingMode, ops.bit, {
                    deref: true
                });
            case 10:
                return vector_1.brk(this._state);
            case 17:
                return this._createAddressing(instruction.addressingMode, (o, s) => (ops.cmp(o, s, state => state.a), null), {
                    deref: true
                });
            case 18:
                return this._createAddressing(instruction.addressingMode, (o, s) => (ops.cmp(o, s, state => state.x), null), {
                    deref: true
                });
            case 19:
                return this._createAddressing(instruction.addressingMode, (o, s) => (ops.cmp(o, s, state => state.y), null), {
                    deref: true
                });
            case 20:
                return this._createAddressing(instruction.addressingMode, instruction_1.readModifyWrite(this._state, (s, o) => ops.genRmw(s, o, x => (x - 1) & 0xff)).reset, {
                    writeOp: true
                });
            case 21:
                return instruction_1.nullaryOneCycle(this._state, s => ops.genNullary(s, state => (state.x = (state.x - 1) & 0xff)));
            case 22:
                return instruction_1.nullaryOneCycle(this._state, s => ops.genNullary(s, state => (state.y = (state.y - 1) & 0xff)));
            case 24:
                return this._createAddressing(instruction.addressingMode, instruction_1.readModifyWrite(this._state, (s, o) => ops.genRmw(s, o, x => (x + 1) & 0xff)).reset, {
                    writeOp: true
                });
            case 25:
                return instruction_1.nullaryOneCycle(this._state, s => ops.genNullary(s, state => (state.x = (state.x + 1) & 0xff)));
            case 26:
                return instruction_1.nullaryOneCycle(this._state, s => ops.genNullary(s, state => (state.y = (state.y + 1) & 0xff)));
            case 23:
                return this._createAddressing(instruction.addressingMode, (o, s) => ops.genUnary(o, s, (operand, state) => (state.a = state.a ^ operand)), {
                    deref: true
                });
            case 27:
                return this._createAddressing(instruction.addressingMode, (o, s) => ((s.p = o), null));
            case 28:
                return instruction_1.jsr(this._state);
            case 29:
                return this._createAddressing(instruction.addressingMode, (o, s) => ops.genUnary(o, s, (operand, state) => (state.a = operand)), {
                    deref: true
                });
            case 30:
                return this._createAddressing(instruction.addressingMode, (o, s) => ops.genUnary(o, s, (operand, state) => (state.x = operand)), {
                    deref: true
                });
            case 31:
                return this._createAddressing(instruction.addressingMode, (o, s) => ops.genUnary(o, s, (operand, state) => (state.y = operand)), {
                    deref: true
                });
            case 32:
                return instruction.addressingMode === 0
                    ? instruction_1.nullaryOneCycle(this._state, ops.lsrImmediate)
                    : this._createAddressing(instruction.addressingMode, instruction_1.readModifyWrite(this._state, ops.lsrRmw).reset, { writeOp: true });
            case 33:
                return instruction_1.nullaryOneCycle(this._state, () => undefined);
            case 34:
                return this._createAddressing(instruction.addressingMode, (o, s) => ops.genUnary(o, s, (operand, state) => (state.a |= operand)), { deref: true });
            case 35:
                return instruction_1.push(this._state, s => s.a);
            case 36:
                return instruction_1.push(this._state, s => s.flags | 16);
            case 37:
                return instruction_1.pull(this._state, (s, o) => ops.genNullary(s, state => (state.a = o)));
            case 38:
                return instruction_1.pull(this._state, (s, o) => (s.flags = (o | 32) & ~16));
            case 39:
                return instruction.addressingMode === 0
                    ? instruction_1.nullaryOneCycle(this._state, ops.rolImmediate)
                    : this._createAddressing(instruction.addressingMode, instruction_1.readModifyWrite(this._state, ops.rolRmw).reset, { writeOp: true });
            case 40:
                return instruction.addressingMode === 0
                    ? instruction_1.nullaryOneCycle(this._state, ops.rorImmediate)
                    : this._createAddressing(instruction.addressingMode, instruction_1.readModifyWrite(this._state, ops.rorRmw).reset, { writeOp: true });
            case 41:
                return instruction_1.rti(this._state);
            case 42:
                return instruction_1.rts(this._state);
            case 43:
                return this._createAddressing(instruction.addressingMode, ops.sbc, {
                    deref: true
                });
            case 48:
                return this._createAddressing(instruction.addressingMode, instruction_1.write(this._state, s => s.x).reset, {
                    writeOp: true
                });
            case 49:
                return this._createAddressing(instruction.addressingMode, instruction_1.write(this._state, s => s.y).reset, {
                    writeOp: true
                });
            case 50:
                return instruction_1.nullaryOneCycle(this._state, s => ops.genNullary(s, state => (state.x = state.a)));
            case 51:
                return instruction_1.nullaryOneCycle(this._state, s => ops.genNullary(s, state => (state.y = state.a)));
            case 52:
                return instruction_1.nullaryOneCycle(this._state, s => ops.genNullary(s, state => (state.x = state.s)));
            case 53:
                return instruction_1.nullaryOneCycle(this._state, s => ops.genNullary(s, state => (state.a = state.x)));
            case 54:
                return instruction_1.nullaryOneCycle(this._state, s => (s.s = s.x));
            case 55:
                return instruction_1.nullaryOneCycle(this._state, s => ops.genNullary(s, state => (state.a = state.y)));
            case 3:
                return instruction_1.branch(this._state, flags => (flags & 1) === 0);
            case 4:
                return instruction_1.branch(this._state, flags => (flags & 1) > 0);
            case 8:
                return instruction_1.branch(this._state, flags => (flags & 2) === 0);
            case 5:
                return instruction_1.branch(this._state, flags => (flags & 2) > 0);
            case 9:
                return instruction_1.branch(this._state, flags => (flags & 128) === 0);
            case 7:
                return instruction_1.branch(this._state, flags => (flags & 128) > 0);
            case 11:
                return instruction_1.branch(this._state, flags => (flags & 64) === 0);
            case 12:
                return instruction_1.branch(this._state, flags => (flags & 64) > 0);
            case 44:
                return instruction_1.nullaryOneCycle(this._state, s => (s.flags |= 1));
            case 45:
                return instruction_1.nullaryOneCycle(this._state, s => (s.flags |= 8));
            case 46:
                return instruction_1.nullaryOneCycle(this._state, s => (s.flags |= 4));
            case 47:
                return this._createAddressing(instruction.addressingMode, instruction_1.write(this._state, s => s.a).reset, {
                    writeOp: true
                });
            case 13:
                return instruction_1.nullaryOneCycle(this._state, s => (s.flags &= ~1));
            case 14:
                return instruction_1.nullaryOneCycle(this._state, s => (s.flags &= ~8));
            case 15:
                return instruction_1.nullaryOneCycle(this._state, s => (s.flags &= ~4));
            case 16:
                return instruction_1.nullaryOneCycle(this._state, s => (s.flags &= ~64));
            case 56:
            case 57:
                return this._createAddressing(instruction.addressingMode, () => null, { deref: true });
            case 67:
                return this._createAddressing(instruction.addressingMode, ops.aac);
            case 64:
                return this._createAddressing(instruction.addressingMode, instruction_1.write(this._state, ops.aax).reset, {
                    writeOp: true
                });
            case 58:
                return this._createAddressing(instruction.addressingMode, ops.alr, {
                    deref: true
                });
            case 62:
                return this._createAddressing(instruction.addressingMode, (o, s) => (ops.arr(o, s), null), {
                    deref: true
                });
            case 59:
                return this._createAddressing(instruction.addressingMode, ops.axs, {
                    deref: true
                });
            case 68:
                return this._createAddressing(instruction.addressingMode, (o, s) => ops.genUnary(o, s, (operand, state) => (state.x = state.a = state.a & operand)), {
                    deref: true
                });
            case 60:
                return this._createAddressing(instruction.addressingMode, instruction_1.readModifyWrite(this._state, ops.dcp).reset, {
                    writeOp: true
                });
            case 66:
                return this._createAddressing(instruction.addressingMode, instruction_1.readModifyWrite(this._state, ops.isc).reset, {
                    writeOp: true
                });
            case 61:
                return this._createAddressing(instruction.addressingMode, (o, s) => ops.genUnary(o, s, (operand, state) => (state.a = state.x = operand)), {
                    deref: true
                });
            case 65:
                return this._createAddressing(instruction.addressingMode, (o, s) => ops.genUnary(o, s, (operand, state) => (state.s = state.x = state.a = state.s & operand)), { deref: true });
            case 70:
                return this._createAddressing(instruction.addressingMode, instruction_1.readModifyWrite(this._state, ops.rla).reset, {
                    writeOp: true
                });
            case 69:
                return this._createAddressing(instruction.addressingMode, instruction_1.readModifyWrite(this._state, ops.rra).reset, {
                    writeOp: true
                });
            case 63:
                return this._createAddressing(instruction.addressingMode, instruction_1.readModifyWrite(this._state, ops.slo).reset, {
                    writeOp: true
                });
            default:
                return null;
        }
    }
    _createAddressing(addressingMode, next, { deref = false, writeOp = false } = {}) {
        if (deref && addressingMode !== 1) {
            next = addressing_1.dereference(this._state, next).reset;
        }
        switch (addressingMode) {
            case 1:
                return addressing_1.immediate(this._state, next);
            case 2:
                return addressing_1.zeroPage(this._state, next);
            case 3:
                return addressing_1.absolute(this._state, next);
            case 6:
                return addressing_1.zeroPageX(this._state, next);
            case 9:
                return addressing_1.zeroPageY(this._state, next);
            case 7:
                return addressing_1.absoluteX(this._state, next, writeOp);
            case 10:
                return addressing_1.absoluteY(this._state, next, writeOp);
            case 8:
                return addressing_1.indexedIndirectX(this._state, next);
            case 11:
                return addressing_1.indirectIndexedY(this._state, next, writeOp);
            case 4:
                return indirect_1.indirect(this._state, next);
            default:
                throw new Error(`invalid addressing mode ${addressingMode}`);
        }
    }
}
exports.default = Compiler;
//# sourceMappingURL=Compiler.js.map