"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.absoluteY = exports.absoluteX = void 0;
const tslib_1 = require("tslib");
const CpuInterface_1 = tslib_1.__importDefault(require("../../CpuInterface"));
const ResultImpl_1 = tslib_1.__importDefault(require("../ResultImpl"));
const decorators_1 = require("../../../../tools/decorators");
class AbsoluteIndexed {
    constructor(state, indexExtractor, next = () => null, writeOp = false) {
        this.reset = () => this._result.read(this._fetchLo, this._state.p);
        this._fetchLo = (value) => {
            this._operand = value;
            this._state.p = (this._state.p + 1) & 0xffff;
            return this._result.read(this._fetchHi, this._state.p);
        };
        this._fetchHi = (value) => {
            this._operand |= value << 8;
            this._state.p = (this._state.p + 1) & 0xffff;
            const index = this._indexExtractor(this._state);
            this._carry = (this._operand & 0xff) + index > 0xff;
            this._operand = (this._operand & 0xff00) | ((this._operand + index) & 0xff);
            return this._carry || this._writeOp
                ? this._result.read(this._dereferenceAndCarry, this._operand)
                : this._next(this._operand, this._state);
        };
        this._dereferenceAndCarry = (value) => {
            if (this._carry) {
                this._operand = (this._operand + 0x0100) & 0xffff;
            }
            return this._next(this._operand, this._state);
        };
        this._operand = 0;
        this._carry = false;
        this._result = new ResultImpl_1.default();
        this._state = state;
        this._indexExtractor = indexExtractor;
        this._next = next;
        this._writeOp = writeOp;
        decorators_1.freezeImmutables(this);
    }
    static absoluteX(state, next, writeOp) {
        return new AbsoluteIndexed(state, s => s.x, next, writeOp);
    }
    static absoluteY(state, next, writeOp) {
        return new AbsoluteIndexed(state, s => s.y, next, writeOp);
    }
}
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], AbsoluteIndexed.prototype, "reset", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], AbsoluteIndexed.prototype, "_fetchLo", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], AbsoluteIndexed.prototype, "_fetchHi", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], AbsoluteIndexed.prototype, "_dereferenceAndCarry", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], AbsoluteIndexed.prototype, "_result", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", CpuInterface_1.default.State)
], AbsoluteIndexed.prototype, "_state", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Function)
], AbsoluteIndexed.prototype, "_indexExtractor", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Function)
], AbsoluteIndexed.prototype, "_next", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Boolean)
], AbsoluteIndexed.prototype, "_writeOp", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [CpuInterface_1.default.State, Function, Boolean]),
    tslib_1.__metadata("design:returntype", AbsoluteIndexed)
], AbsoluteIndexed, "absoluteX", null);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [CpuInterface_1.default.State, Function, Boolean]),
    tslib_1.__metadata("design:returntype", AbsoluteIndexed)
], AbsoluteIndexed, "absoluteY", null);
exports.absoluteX = (state, next, writeOp) => AbsoluteIndexed.absoluteX(state, next, writeOp);
exports.absoluteY = (state, next, writeOp) => AbsoluteIndexed.absoluteY(state, next, writeOp);
//# sourceMappingURL=absoluteIndexed.js.map