"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.jsr = void 0;
const tslib_1 = require("tslib");
const CpuInterface_1 = tslib_1.__importDefault(require("../../CpuInterface"));
const ResultImpl_1 = tslib_1.__importDefault(require("../ResultImpl"));
const decorators_1 = require("../../../../tools/decorators");
class Jsr {
    constructor(state) {
        this.reset = () => this._result.read(this._fetchPcl, this._state.p);
        this._fetchPcl = (value) => {
            this._addressLo = value;
            this._state.p = (this._state.p + 1) & 0xffff;
            return this._result.read(this._dummyStackRead, 0x0100 + this._state.s);
        };
        this._dummyStackRead = () => this._result.write(this._pushPch, 0x0100 + this._state.s, this._state.p >>> 8);
        this._pushPch = () => {
            this._state.s = (this._state.s - 1) & 0xff;
            return this._result.write(this._pushPcl, 0x0100 + this._state.s, this._state.p & 0xff);
        };
        this._pushPcl = () => {
            this._state.s = (this._state.s - 1) & 0xff;
            return this._result.read(this._fetchPch, this._state.p);
        };
        this._fetchPch = (value) => {
            this._state.p = this._addressLo | (value << 8);
            return null;
        };
        this._addressLo = 0;
        this._result = new ResultImpl_1.default();
        this._state = state;
        decorators_1.freezeImmutables(this);
    }
}
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Jsr.prototype, "_fetchPcl", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Jsr.prototype, "_dummyStackRead", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Jsr.prototype, "_pushPch", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Jsr.prototype, "_pushPcl", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Jsr.prototype, "_fetchPch", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Jsr.prototype, "_result", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", CpuInterface_1.default.State)
], Jsr.prototype, "_state", void 0);
exports.jsr = (state) => new Jsr(state);
//# sourceMappingURL=jsr.js.map