"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.aac = exports.isc = exports.aax = exports.slo = exports.rla = exports.rra = exports.axs = exports.dcp = exports.alr = exports.arr = exports.rorRmw = exports.rorImmediate = exports.rolRmw = exports.rolImmediate = exports.lsrRmw = exports.lsrImmediate = exports.sbc = exports.cmp = exports.bit = exports.aslRmw = exports.aslImmediate = exports.adc = exports.genUnary = exports.genNullary = exports.genRmw = void 0;
function setFlagsNZ(operand, state) {
    state.flags =
        (state.flags & ~(128 | 2)) |
            (operand & 0x80) |
            (operand ? 0 : 2);
}
function genRmw(operand, state, operation) {
    const result = operation(operand);
    setFlagsNZ(result, state);
    return result;
}
exports.genRmw = genRmw;
function genNullary(state, operation) {
    setFlagsNZ(operation(state), state);
}
exports.genNullary = genNullary;
function genUnary(operand, state, operation) {
    setFlagsNZ(operation(operand, state), state);
    return null;
}
exports.genUnary = genUnary;
function adc(operand, state) {
    if (state.flags & 8) {
        const d0 = (operand & 0x0f) + (state.a & 0x0f) + (state.flags & 1), d1 = (operand >>> 4) + (state.a >>> 4) + (d0 > 9 ? 1 : 0);
        state.a = d0 % 10 | (d1 % 10 << 4);
        state.flags =
            (state.flags & ~(128 | 2 | 1)) |
                (state.a & 0x80) |
                (state.a ? 0 : 2) |
                (d1 > 9 ? 1 : 0);
    }
    else {
        const sum = state.a + operand + (state.flags & 1), result = sum & 0xff;
        state.flags =
            (state.flags &
                ~(128 | 2 | 1 | 64)) |
                (result & 0x80) |
                (result ? 0 : 2) |
                (sum >>> 8) |
                ((~(operand ^ state.a) & (result ^ operand) & 0x80) >>> 1);
        state.a = result;
    }
    return null;
}
exports.adc = adc;
function aslImmediate(state) {
    const old = state.a;
    state.a = (state.a << 1) & 0xff;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (state.a & 0x80) |
            (state.a ? 0 : 2) |
            (old >>> 7);
}
exports.aslImmediate = aslImmediate;
function aslRmw(operand, state) {
    const result = (operand << 1) & 0xff;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (result & 0x80) |
            (result ? 0 : 2) |
            (operand >>> 7);
    return result;
}
exports.aslRmw = aslRmw;
function bit(operand, state) {
    state.flags =
        (state.flags & ~(128 | 64 | 2)) |
            (operand & (128 | 64)) |
            (operand & state.a ? 0 : 2);
    return null;
}
exports.bit = bit;
function cmp(operand, state, getRegister) {
    const diff = getRegister(state) + (~operand & 0xff) + 1;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (diff & 0x80) |
            (diff & 0xff ? 0 : 2) |
            (diff >>> 8);
}
exports.cmp = cmp;
function sbc(operand, state) {
    if (state.flags & 8) {
        const d0 = (state.a & 0x0f) - (operand & 0x0f) - (~state.flags & 1), d1 = (state.a >>> 4) - (operand >>> 4) - (d0 < 0 ? 1 : 0);
        state.a = (d0 < 0 ? 10 + d0 : d0) | ((d1 < 0 ? 10 + d1 : d1) << 4);
        state.flags =
            (state.flags & ~(128 | 2 | 1)) |
                (state.a & 0x80) |
                (state.a ? 0 : 2) |
                (d1 < 0 ? 0 : 1);
    }
    else {
        operand = ~operand & 0xff;
        const sum = state.a + operand + (state.flags & 1), result = sum & 0xff;
        state.flags =
            (state.flags &
                ~(128 | 2 | 1 | 64)) |
                (result & 0x80) |
                (result ? 0 : 2) |
                (sum >>> 8) |
                ((~(operand ^ state.a) & (result ^ operand) & 0x80) >>> 1);
        state.a = result;
    }
    return null;
}
exports.sbc = sbc;
function lsrImmediate(state) {
    const old = state.a;
    state.a = state.a >>> 1;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (state.a & 0x80) |
            (state.a ? 0 : 2) |
            (old & 1);
}
exports.lsrImmediate = lsrImmediate;
function lsrRmw(operand, state) {
    const result = operand >>> 1;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (result & 0x80) |
            (result ? 0 : 2) |
            (operand & 1);
    return result;
}
exports.lsrRmw = lsrRmw;
function rolImmediate(state) {
    const old = state.a;
    state.a = ((state.a << 1) & 0xff) | (state.flags & 1);
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (state.a & 0x80) |
            (state.a ? 0 : 2) |
            (old >>> 7);
}
exports.rolImmediate = rolImmediate;
function rolRmw(operand, state) {
    const result = ((operand << 1) & 0xff) | (state.flags & 1);
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (result & 0x80) |
            (result ? 0 : 2) |
            (operand >>> 7);
    return result;
}
exports.rolRmw = rolRmw;
function rorImmediate(state) {
    const old = state.a;
    state.a = (state.a >>> 1) | ((state.flags & 1) << 7);
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (state.a & 0x80) |
            (state.a ? 0 : 2) |
            (old & 1);
}
exports.rorImmediate = rorImmediate;
function rorRmw(operand, state) {
    const result = (operand >>> 1) | ((state.flags & 1) << 7);
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (result & 0x80) |
            (result ? 0 : 2) |
            (operand & 1);
    return result;
}
exports.rorRmw = rorRmw;
function arr(operand, state) {
    state.a = ((state.a & operand) >>> 1) | (state.flags & 1 ? 0x80 : 0);
    state.flags =
        (state.flags & ~(1 | 128 | 2 | 64)) |
            ((state.a & 0x40) >>> 6) |
            (state.a ? 0 : 2) |
            (state.a & 0x80) |
            ((state.a & 0x40) ^ ((state.a & 0x20) << 1));
}
exports.arr = arr;
function alr(operand, state) {
    const i = state.a & operand;
    state.a = i >>> 1;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (state.a & 0x80) |
            (state.a ? 0 : 2) |
            (i & 1);
    return null;
}
exports.alr = alr;
function dcp(operand, state) {
    const result = (operand + 0xff) & 0xff;
    const diff = state.a + (~result & 0xff) + 1;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (diff & 0x80) |
            (diff & 0xff ? 0 : 2) |
            (diff >>> 8);
    return result;
}
exports.dcp = dcp;
function axs(operand, state) {
    const value = (state.a & state.x) + (~operand & 0xff) + 1;
    state.x = value & 0xff;
    state.flags =
        (state.flags & ~(128 | 2 | 1)) |
            (state.x & 0x80) |
            (state.x & 0xff ? 0 : 2) |
            (value >>> 8);
    return null;
}
exports.axs = axs;
function rra(operand, state) {
    const result = (operand >>> 1) | ((state.flags & 1) << 7);
    state.flags = (state.flags & ~1) | (operand & 1);
    adc(result, state);
    return result;
}
exports.rra = rra;
function rla(operand, state) {
    const result = ((operand << 1) & 0xff) | (state.flags & 1);
    state.flags = (state.flags & ~1) | (operand >>> 7);
    setFlagsNZ((state.a &= result), state);
    return result;
}
exports.rla = rla;
function slo(operand, state) {
    state.flags = (state.flags & ~1) | (operand >>> 7);
    const result = (operand << 1) & 0xff;
    state.a = state.a | result;
    setFlagsNZ(state.a, state);
    return result;
}
exports.slo = slo;
function aax(state) {
    const result = state.a & state.x;
    setFlagsNZ(result, state);
    return result;
}
exports.aax = aax;
function isc(operand, state) {
    const result = (operand + 1) & 0xff;
    sbc(result, state);
    return result;
}
exports.isc = isc;
function aac(operand, state) {
    state.a &= operand;
    setFlagsNZ(state.a, state);
    state.flags = (state.flags & ~1) | ((state.a & 0x80) >>> 7);
    return null;
}
exports.aac = aac;
//# sourceMappingURL=ops.js.map