"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.boot = void 0;
const tslib_1 = require("tslib");
const CpuInterface_1 = tslib_1.__importDefault(require("../../CpuInterface"));
const ResultImpl_1 = tslib_1.__importDefault(require("../ResultImpl"));
const decorators_1 = require("../../../../tools/decorators");
class Boot {
    constructor(state) {
        this.reset = () => this._result.read(this._pre1Step, 0xff);
        this._pre1Step = () => this._result.read(this._pre2Step, 0x0ff);
        this._pre2Step = () => this._result.read(this._stack1Step, 0x0100);
        this._stack1Step = () => this._result.read(this._stack2Step, 0x01ff);
        this._stack2Step = () => {
            this._state.s = 0xfd;
            return this._result.read(this._stack3Step, 0x01fe);
        };
        this._stack3Step = () => this._result.read(this._readTargetLoStep, 0xfffc);
        this._readTargetLoStep = (operand) => {
            this._targetAddress = operand;
            return this._result.read(this._readTargetHiStep, 0xfffd);
        };
        this._readTargetHiStep = (operand) => {
            this._targetAddress |= operand << 8;
            this._state.p = this._targetAddress;
            return null;
        };
        this._targetAddress = 0;
        this._result = new ResultImpl_1.default();
        this._state = state;
        decorators_1.freezeImmutables(this);
    }
}
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Boot.prototype, "reset", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Boot.prototype, "_pre1Step", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Boot.prototype, "_pre2Step", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Boot.prototype, "_stack1Step", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Boot.prototype, "_stack2Step", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Boot.prototype, "_stack3Step", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Boot.prototype, "_readTargetLoStep", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Boot.prototype, "_readTargetHiStep", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Boot.prototype, "_result", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", CpuInterface_1.default.State)
], Boot.prototype, "_state", void 0);
exports.boot = (state) => new Boot(state);
//# sourceMappingURL=boot.js.map