"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.nmi = exports.irq = exports.brk = void 0;
const tslib_1 = require("tslib");
const CpuInterface_1 = tslib_1.__importDefault(require("../../CpuInterface"));
const ResultImpl_1 = tslib_1.__importDefault(require("../ResultImpl"));
const decorators_1 = require("../../../../tools/decorators");
class Interrupt {
    constructor(state, defaultVector, isBrk) {
        this.reset = () => this._result.read(this._dummyRead, this._state.p);
        this._dummyRead = () => {
            if (this._isBrk) {
                this._state.p = (this._state.p + 1) & 0xffff;
            }
            return this._result.write(this._pushPch, 0x0100 + this._state.s, this._state.p >>> 8);
        };
        this._pushPch = () => {
            this._state.s = (this._state.s - 1) & 0xff;
            return this._result.write(this._pushPcl, 0x0100 + this._state.s, this._state.p & 0xff).poll(true);
        };
        this._pushPcl = () => {
            this._state.s = (this._state.s - 1) & 0xff;
            this._vector = this._state.nmi ? 0xfffa : this._defaultVector;
            return this._result.write(this._pushFlags, 0x0100 + this._state.s, this._isBrk ? this._state.flags | 16 : this._state.flags & ~16);
        };
        this._pushFlags = () => {
            this._state.s = (this._state.s - 1) & 0xff;
            return this._result.read(this._fetchPcl, this._vector);
        };
        this._fetchPcl = (value) => {
            this._state.flags |= 4;
            this._state.p = value;
            return this._result.read(this._fetchPch, ++this._vector);
        };
        this._fetchPch = (value) => {
            this._state.p = this._state.p | (value << 8);
            this._state.nmi = this._state.irq = false;
            return null;
        };
        this._vector = 0;
        this._result = new ResultImpl_1.default();
        this._state = state;
        this._defaultVector = defaultVector;
        this._isBrk = isBrk;
        decorators_1.freezeImmutables(this);
    }
}
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Interrupt.prototype, "reset", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Interrupt.prototype, "_dummyRead", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Interrupt.prototype, "_pushPch", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Interrupt.prototype, "_pushPcl", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Interrupt.prototype, "_pushFlags", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Interrupt.prototype, "_fetchPcl", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Interrupt.prototype, "_fetchPch", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Object)
], Interrupt.prototype, "_result", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", CpuInterface_1.default.State)
], Interrupt.prototype, "_state", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Number)
], Interrupt.prototype, "_defaultVector", void 0);
tslib_1.__decorate([
    decorators_1.Immutable,
    tslib_1.__metadata("design:type", Boolean)
], Interrupt.prototype, "_isBrk", void 0);
exports.brk = (state) => new Interrupt(state, 0xfffe, true);
exports.irq = (state) => new Interrupt(state, 0xfffe, false);
exports.nmi = (state) => new Interrupt(state, 0xfffa, false);
//# sourceMappingURL=interrupt.js.map