"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const Memory_1 = tslib_1.__importDefault(require("../vanilla/Memory"));
class Memory extends Memory_1.default {
    constructor() {
        super(...arguments);
        this._inCallback = () => 0x00;
        this._outCallback = () => undefined;
        this._feedbackRegister = 0;
    }
    reset() {
        super.reset();
        this._feedbackRegister = 0;
    }
    setCpu(cpu) {
        this._cpu = cpu;
        return this;
    }
    read(address) {
        switch (address) {
            case 0xf002:
                return this._feedbackRegister;
            case 0xf004:
                return this._inCallback(this);
            default:
                return this._data[address];
        }
    }
    readWord(address) {
        if ((address & 0xfff0) === 0xf000) {
            return this.read(address) + (this.read((address + 1) & 0xffff) << 8);
        }
        return this._data[address] + (this._data[(address + 1) & 0xffff] << 8);
    }
    write(address, value) {
        switch (address) {
            case 0xf001:
                this._outCallback(value, this);
                break;
            case 0xf002:
                this._cpu.setInterrupt(!!(value & 0x01));
                if (value & 0x02 && !(this._feedbackRegister & 0x02)) {
                    this._cpu.nmi();
                }
                this._feedbackRegister = value;
                break;
            default:
                if (address < 0xc000) {
                    this._data[address] = value;
                }
                break;
        }
    }
    setInCallback(callback) {
        this._inCallback = callback;
        return this;
    }
    getInCallback() {
        return this._inCallback;
    }
    setOutCallback(callback) {
        this._outCallback = callback;
        return this;
    }
    getOutCallback() {
        return this._outCallback;
    }
}
exports.default = Memory;
//# sourceMappingURL=Memory.js.map