"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const microevent_ts_1 = require("microevent.ts");
class AsyncIO {
    constructor(_board) {
        this._board = _board;
        this.message = new microevent_ts_1.Event();
        this._bufferOut = new Uint8Array(256);
        this._bufferIn = new Uint8Array(256);
        this._indexBufferOut = 0;
        this._indexBufferIn = -1;
        this._bus = null;
        this._bus = this._board.getBus();
        this._board.systemReset.addHandler(AsyncIO._onReset, this);
        this._bus.event.write.addHandler(AsyncIO._onWrite, this);
        this._bus.event.read.addHandler(AsyncIO._onRead, this);
        this.reset();
    }
    reset() {
        for (let i = 0; i < this._bufferOut.length; i++) {
            this._bufferOut[i] = 0;
        }
        this._indexBufferOut = 0;
        return this;
    }
    send(message) {
        this._indexBufferIn = message.length > 256 ? 255 : message.length - 1;
        for (let i = 0; i <= this._indexBufferIn; i++) {
            this._bufferIn[i] = message[i];
        }
    }
    static _onWrite(accessType, self) {
        if (accessType !== 0) {
            return;
        }
        switch (self._bus.getLastAddresBusValue()) {
            case 0x30:
                self._bufferOut[self._indexBufferOut] = self._bus.getLastDataBusValue();
                self._indexBufferOut = (self._indexBufferOut + 1) % self._bufferOut.length;
                break;
            case 0x31: {
                const count = self._bus.getLastDataBusValue();
                const data = new Uint8Array(count);
                for (let i = 0; i < count; i++) {
                    const j = (self._indexBufferOut - 1 + self._bufferOut.length) % self._bufferOut.length;
                    data[count - i - 1] = self._bufferOut[j];
                    self._indexBufferOut = j;
                }
                self.message.dispatch(data);
                break;
            }
        }
    }
    static _onRead(accessType, self) {
        if (accessType !== 0) {
            return;
        }
        switch (self._bus.getLastAddresBusValue()) {
            case 0x30:
                self._bus.setDataBusValue(self._indexBufferIn >= 0 ? self._bufferIn[self._indexBufferIn--] : 0);
                break;
            case 0x31: {
                self._bus.setDataBusValue(self._indexBufferIn >= 0 ? self._indexBufferIn + 1 : 0);
                break;
            }
        }
    }
    static _onReset(payload, self) {
        self.reset();
    }
}
exports.default = AsyncIO;
//# sourceMappingURL=AsyncIO.js.map