"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const microevent_ts_1 = require("microevent.ts");
class Bus {
    constructor() {
        this.event = {
            trap: new microevent_ts_1.Event(),
            read: new microevent_ts_1.Event(),
            write: new microevent_ts_1.Event()
        };
        this._tia = null;
        this._pia = null;
        this._cartridge = null;
        this._lastDataBusValue = 0;
        this._lastAddressBusValue = 0;
    }
    setTia(tia) {
        tia.trap.addHandler((payload) => this.triggerTrap(0, 'TIA: ' + (payload.message || '')));
        this._tia = tia;
        return this;
    }
    setPia(pia) {
        pia.trap.addHandler((payload) => this.triggerTrap(1, 'PIA: ' + (payload.message || '')));
        this._pia = pia;
        return this;
    }
    setCartridge(cartridge) {
        cartridge.trap.addHandler((payload) => this.triggerTrap(2, 'CARTRIDGE: ' + (payload.message || '')));
        this._cartridge = cartridge;
        return this;
    }
    readWord(address) {
        return this.read(address) | (this.read((address + 1) & 0xffff) << 8);
    }
    read(address) {
        this._lastAddressBusValue = address;
        address &= 0x1fff;
        if (address & 0x1000) {
            this._lastDataBusValue = this._cartridge.read(address);
            this.event.read.dispatch(2);
        }
        else if (address & 0x80) {
            this._lastDataBusValue = this._pia.read(address);
            this.event.read.dispatch(1);
        }
        else {
            this._lastDataBusValue = this._tia.read(address);
            this.event.read.dispatch(0);
        }
        return this._lastDataBusValue;
    }
    write(address, value) {
        this._lastDataBusValue = value;
        this._lastAddressBusValue = address;
        address &= 0x1fff;
        if (address & 0x1000) {
            this._cartridge.write(address, value);
            this.event.write.dispatch(2);
        }
        else if (address & 0x80) {
            this._pia.write(address, value);
            this.event.write.dispatch(1);
        }
        else {
            this._tia.write(address, value);
            this.event.write.dispatch(0);
        }
    }
    peek(address) {
        address &= 0x1fff;
        if (address & 0x1000) {
            return this._cartridge.peek(address);
        }
        else if (address & 0x80) {
            return this._pia.peek(address);
        }
        else {
            return this._tia.peek(address);
        }
    }
    poke(address, value) { }
    getLastDataBusValue() {
        return this._lastDataBusValue;
    }
    setDataBusValue(value) {
        this._lastDataBusValue = value;
    }
    getLastAddresBusValue() {
        return this._lastAddressBusValue;
    }
    triggerTrap(reason, message) {
        if (this.event.trap.hasHandlers) {
            this.event.trap.dispatch(new Bus.TrapPayload(reason, this, message));
        }
        else {
            throw new Error(message);
        }
    }
}
exports.default = Bus;
(function (Bus) {
    class TrapPayload {
        constructor(reason, bus, message) {
            this.reason = reason;
            this.bus = bus;
            this.message = message;
        }
    }
    Bus.TrapPayload = TrapPayload;
})(Bus || (Bus = {}));
exports.default = Bus;
//# sourceMappingURL=Bus.js.map