"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const AbstractCartridge_1 = tslib_1.__importDefault(require("./AbstractCartridge"));
const CartridgeInfo_1 = tslib_1.__importDefault(require("./CartridgeInfo"));
const cartridgeUtil = tslib_1.__importStar(require("./util"));
class Cartridge8040 extends AbstractCartridge_1.default {
    constructor(buffer) {
        super();
        this._bank = null;
        this._bank0 = new Uint8Array(0x1000);
        this._bank1 = new Uint8Array(0x1000);
        this._bus = null;
        if (buffer.length !== 0x2000) {
            throw new Error(`buffer is not an 8k cartridge image: wrong length ${buffer.length}`);
        }
        for (let i = 0; i < 0x1000; i++) {
            this._bank0[i] = buffer[i];
            this._bank1[i] = buffer[0x1000 + i];
        }
        this.reset();
    }
    static matchesBuffer(buffer) {
        const signatureCounts = cartridgeUtil.searchForSignatures(buffer, [
            [0xad, 0x00, 0x08],
            [0xad, 0x40, 0x08],
            [0x2c, 0x00, 0x08],
            [0x0c, 0x00, 0x08, 0x4c],
            [0x0c, 0xff, 0x0f, 0x4c]
        ]);
        for (const count of signatureCounts) {
            if (count >= 2) {
                return true;
            }
        }
        return false;
    }
    reset() {
        this._bank = this._bank0;
    }
    read(address) {
        return this.peek(address);
    }
    peek(address) {
        address &= 0x0fff;
        return this._bank[address];
    }
    getType() {
        return CartridgeInfo_1.default.CartridgeType.bankswitch_8k_econobanking;
    }
    setBus(bus) {
        this._bus = bus;
        this._bus.event.read.addHandler(this._onBusAccess, this);
        this._bus.event.write.addHandler(this._onBusAccess, this);
        return this;
    }
    _onBusAccess(accessType, self) {
        const address = self._bus.getLastAddresBusValue() & 0x1840;
        switch (address) {
            case 0x0800:
                self._bank = self._bank0;
                break;
            case 0x0840:
                self._bank = self._bank1;
                break;
        }
    }
}
exports.default = Cartridge8040;
//# sourceMappingURL=Cartridge0840.js.map