"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const AbstractCartridge_1 = tslib_1.__importDefault(require("./AbstractCartridge"));
const CartridgeInfo_1 = tslib_1.__importDefault(require("./CartridgeInfo"));
function nextPowerOfTwo(x) {
    let v = 1;
    while (v < x) {
        v *= 2;
    }
    return v;
}
function padBuffer(buffer) {
    const paddedLength = nextPowerOfTwo(buffer.length);
    if (paddedLength === buffer.length) {
        return buffer;
    }
    const paddedBuffer = new Uint8Array(paddedLength);
    for (let i = 0; i < paddedLength; i++) {
        paddedBuffer[paddedLength - i - 1] = i < buffer.length ? buffer[buffer.length - i - 1] : 0;
    }
    return paddedBuffer;
}
class Cartridge2k extends AbstractCartridge_1.default {
    constructor(buffer) {
        super();
        this._rom = new Uint8Array(0x0800);
        if (buffer.length > 0x0800) {
            throw new Error(`buffer is not a 2k cartridge image: wrong length ${buffer.length}`);
        }
        const paddedBuffer = padBuffer(buffer);
        for (let i = 0; i < 0x0800; i++) {
            this._rom[i] = buffer[i % paddedBuffer.length];
        }
    }
    read(address) {
        return this._rom[address & 0x07ff];
    }
    getType() {
        return CartridgeInfo_1.default.CartridgeType.vanilla_2k;
    }
}
exports.default = Cartridge2k;
//# sourceMappingURL=Cartridge2k.js.map