"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const AbstractCartridge_1 = tslib_1.__importDefault(require("./AbstractCartridge"));
const cartridgeUtil = tslib_1.__importStar(require("./util"));
const CartridgeInfo_1 = tslib_1.__importDefault(require("./CartridgeInfo"));
class Cartridge3F extends AbstractCartridge_1.default {
    constructor(buffer) {
        super();
        this._banks = new Array(4);
        this._bus = null;
        if (buffer.length !== 0x2000) {
            throw new Error(`buffer is not an 8k cartridge image: invalid length ${buffer.length}`);
        }
        for (let i = 0; i < 4; i++) {
            this._banks[i] = new Uint8Array(0x0800);
        }
        this._bank1 = this._banks[3];
        this._bank0 = this._banks[0];
        for (let i = 0; i < 0x0800; i++) {
            for (let j = 0; j < 4; j++) {
                this._banks[j][i] = buffer[0x0800 * j + i];
            }
        }
    }
    static matchesBuffer(buffer) {
        const signatureCounts = cartridgeUtil.searchForSignatures(buffer, [[0x85, 0x3f]]);
        return signatureCounts[0] >= 2;
    }
    reset() {
        this._bank0 = this._banks[0];
    }
    setBus(bus) {
        this._bus = bus;
        this._bus.event.read.addHandler(Cartridge3F._onBusAccess, this);
        this._bus.event.write.addHandler(Cartridge3F._onBusAccess, this);
        return this;
    }
    read(address) {
        address &= 0x0fff;
        return address < 0x0800 ? this._bank0[address] : this._bank1[address & 0x07ff];
    }
    getType() {
        return CartridgeInfo_1.default.CartridgeType.bankswitch_8k_3F;
    }
    static _onBusAccess(accessType, self) {
        if (self._bus.getLastAddresBusValue() === 0x003f) {
            self._bank0 = self._banks[self._bus.getLastDataBusValue() & 0x03];
        }
    }
}
exports.default = Cartridge3F;
//# sourceMappingURL=Cartridge3F.js.map