"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const AbstractCartridge_1 = tslib_1.__importDefault(require("./AbstractCartridge"));
const CartridgeInfo_1 = tslib_1.__importDefault(require("./CartridgeInfo"));
const util_1 = require("./util");
class CartridgeCV extends AbstractCartridge_1.default {
    constructor(buffer) {
        super();
        this._rom = new Uint8Array(0x0800);
        this._ram = new Uint8Array(0x0400);
        if (buffer.length !== 0x0800) {
            throw new Error(`buffer is not a 2k cartridge image: wrong length ${buffer.length}`);
        }
        for (let i = 0; i < 0x0800; i++) {
            this._rom[i] = buffer[i];
        }
    }
    static matchesBuffer(buffer) {
        const signatureCounts = util_1.searchForSignatures(buffer, [[0x9d, 0xff, 0xf3], [0x99, 0x00, 0xf4]]);
        return signatureCounts[0] > 0 || signatureCounts[1] > 0;
    }
    setBus(bus) {
        this._bus = bus;
        return this;
    }
    randomize(rng) {
        for (let i = 0; i < 0x0400; i++) {
            this._ram[i] = rng.int(0xff);
        }
    }
    read(address) {
        address &= 0x0fff;
        if (address < 0x0400) {
            return this._ram[address];
        }
        if (address < 0x0800) {
            return (this._ram[address & 0x03ff] = this._bus.getLastDataBusValue());
        }
        return this._rom[address & 0x07ff];
    }
    write(address, value) {
        address &= 0x0fff;
        if (address >= 0x0400 && address < 0x0800) {
            this._ram[address & 0x03ff] = value;
        }
    }
    peek(address) {
        address &= 0x0fff;
        if (address < 0x0400) {
            return this._ram[address];
        }
        if (address < 0x0800) {
            return 0;
        }
        return this._rom[address & 0x07ff];
    }
    getType() {
        return CartridgeInfo_1.default.CartridgeType.bankswitch_2k_cv;
    }
}
exports.default = CartridgeCV;
//# sourceMappingURL=CartridgeCV.js.map