"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const CartridgeInfo_1 = tslib_1.__importDefault(require("./CartridgeInfo"));
const CartridgeF8_1 = tslib_1.__importDefault(require("./CartridgeF8"));
const CartridgeE0_1 = tslib_1.__importDefault(require("./CartridgeE0"));
const Cartridge3F_1 = tslib_1.__importDefault(require("./Cartridge3F"));
const Cartridge3E_1 = tslib_1.__importDefault(require("./Cartridge3E"));
const CartridgeFE_1 = tslib_1.__importDefault(require("./CartridgeFE"));
const CartridgeUA_1 = tslib_1.__importDefault(require("./CartridgeUA"));
const CartridgeE7_1 = tslib_1.__importDefault(require("./CartridgeE7"));
const CartridgeFA2_1 = tslib_1.__importDefault(require("./CartridgeFA2"));
const CartridgeEF_1 = tslib_1.__importDefault(require("./CartridgeEF"));
const CartridgeDPCPlus_1 = tslib_1.__importDefault(require("./CartridgeDPCPlus"));
const CartridgeCDF_1 = tslib_1.__importDefault(require("./CartridgeCDF"));
const Cartridge0840_1 = tslib_1.__importDefault(require("./Cartridge0840"));
const CartridgeCV_1 = tslib_1.__importDefault(require("./CartridgeCV"));
class CartridgeDetector {
    detectCartridgeType(buffer) {
        if (buffer.length % 8448 === 0) {
            return CartridgeInfo_1.default.CartridgeType.bankswitch_supercharger;
        }
        if (buffer.length < 0x0800) {
            return CartridgeInfo_1.default.CartridgeType.vanilla_2k;
        }
        if (buffer.length >= 10240 && buffer.length <= 10496) {
            return CartridgeInfo_1.default.CartridgeType.bankswitch_8k_DPC;
        }
        switch (buffer.length) {
            case 0x0800:
                return this._detect2k(buffer);
            case 0x1000:
                return CartridgeInfo_1.default.CartridgeType.vanilla_4k;
            case 0x2000:
                return this._detect8k(buffer);
            case 0x2003:
                return CartridgeInfo_1.default.CartridgeType.bankswitch_8k_pp;
            case 0x3000:
                return CartridgeInfo_1.default.CartridgeType.bankswitch_12k_FA;
            case 0x4000:
                return this._detect16k(buffer);
            case 0x7000:
                return CartridgeInfo_1.default.CartridgeType.bankswitch_FA2;
            case 0x7400:
                return this._detect29k(buffer);
            case 0x8000:
                return this._detect32k(buffer);
            case 0x10000:
                return this._detect64k(buffer);
            default:
                return CartridgeInfo_1.default.CartridgeType.unknown;
        }
    }
    _detect2k(buffer) {
        if (CartridgeCV_1.default.matchesBuffer(buffer)) {
            return CartridgeInfo_1.default.CartridgeType.bankswitch_2k_cv;
        }
        return CartridgeInfo_1.default.CartridgeType.vanilla_2k;
    }
    _detect8k(buffer) {
        const f8Matches = CartridgeF8_1.default.matchesBuffer(buffer);
        if (CartridgeE0_1.default.matchesBuffer(buffer)) {
            return CartridgeInfo_1.default.CartridgeType.bankswitch_8k_E0;
        }
        if (Cartridge3F_1.default.matchesBuffer(buffer)) {
            return CartridgeInfo_1.default.CartridgeType.bankswitch_8k_3F;
        }
        if (CartridgeUA_1.default.matchesBuffer(buffer)) {
            return CartridgeInfo_1.default.CartridgeType.bankswitch_8k_UA;
        }
        if (!f8Matches && CartridgeFE_1.default.matchesBuffer(buffer)) {
            return CartridgeInfo_1.default.CartridgeType.bankswitch_8k_FE;
        }
        if (Cartridge0840_1.default.matchesBuffer(buffer)) {
            return CartridgeInfo_1.default.CartridgeType.bankswitch_8k_econobanking;
        }
        return CartridgeInfo_1.default.CartridgeType.bankswitch_8k_F8;
    }
    _detect16k(buffer) {
        if (CartridgeE7_1.default.matchesBuffer(buffer)) {
            return CartridgeInfo_1.default.CartridgeType.bankswitch_16k_E7;
        }
        return CartridgeInfo_1.default.CartridgeType.bankswitch_16k_F6;
    }
    _detect29k(buffer) {
        if (CartridgeFA2_1.default.matchesBuffer(buffer)) {
            return CartridgeInfo_1.default.CartridgeType.bankswitch_FA2;
        }
        return CartridgeInfo_1.default.CartridgeType.bankswitch_dpc_plus;
    }
    _detect32k(buffer) {
        if (Cartridge3E_1.default.matchesBuffer(buffer)) {
            return CartridgeInfo_1.default.CartridgeType.bankswitch_3E;
        }
        if (CartridgeDPCPlus_1.default.matchesBuffer(buffer)) {
            return CartridgeInfo_1.default.CartridgeType.bankswitch_dpc_plus;
        }
        if (CartridgeCDF_1.default.matchesBuffer(buffer)) {
            return CartridgeInfo_1.default.CartridgeType.bankswitch_cdf;
        }
        return CartridgeInfo_1.default.CartridgeType.bankswitch_32k_F4;
    }
    _detect64k(buffer) {
        if (CartridgeEF_1.default.matchesBuffer(buffer)) {
            return CartridgeInfo_1.default.CartridgeType.bankswitch_64k_EF;
        }
        return CartridgeInfo_1.default.CartridgeType.bankswitch_64k_F0;
    }
}
exports.default = CartridgeDetector;
//# sourceMappingURL=CartridgeDetector.js.map