"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const AbstractCartridge_1 = tslib_1.__importDefault(require("./AbstractCartridge"));
const cartridgeUtil = tslib_1.__importStar(require("./util"));
const CartridgeInfo_1 = tslib_1.__importDefault(require("./CartridgeInfo"));
class CartridgeE0 extends AbstractCartridge_1.default {
    constructor(buffer) {
        super();
        this._banks = new Array(8);
        this._activeBanks = new Array(4);
        if (buffer.length !== 0x2000) {
            throw new Error(`buffer is not an 8k cartridge image: invalid length ${buffer.length}`);
        }
        for (let i = 0; i < 8; i++) {
            this._banks[i] = new Uint8Array(0x0400);
        }
        for (let i = 0; i < 0x0400; i++) {
            for (let j = 0; j < 8; j++) {
                this._banks[j][i] = buffer[j * 0x0400 + i];
            }
        }
        this.reset();
    }
    static matchesBuffer(buffer) {
        const signatureCounts = cartridgeUtil.searchForSignatures(buffer, [
            [0x8d, 0xe0, 0x1f],
            [0x8d, 0xe0, 0x5f],
            [0x8d, 0xe9, 0xff],
            [0x0c, 0xe0, 0x1f],
            [0xad, 0xe0, 0x1f],
            [0xad, 0xe9, 0xff],
            [0xad, 0xed, 0xff],
            [0xad, 0xf3, 0xbf]
        ]);
        for (let i = 0; i < signatureCounts.length; i++) {
            if (signatureCounts[i] > 0) {
                return true;
            }
        }
        return false;
    }
    reset() {
        for (let i = 0; i < 4; i++) {
            this._activeBanks[i] = this._banks[7];
        }
    }
    read(address) {
        address &= 0x0fff;
        if (address >= 0x0fe0 && address < 0x0ff8) {
            this._handleBankswitch(address);
        }
        return this._activeBanks[address >> 10][address & 0x03ff];
    }
    peek(address) {
        address &= 0x0fff;
        return this._activeBanks[address >> 10][address & 0x03ff];
    }
    write(address, value) {
        const addressMasked = address & 0x0fff;
        if (addressMasked >= 0x0fe0 && addressMasked < 0x0ff8) {
            this._handleBankswitch(addressMasked);
        }
        return super.write(address, value);
    }
    getType() {
        return CartridgeInfo_1.default.CartridgeType.bankswitch_8k_E0;
    }
    _handleBankswitch(address) {
        if (address < 0x0fe8) {
            this._activeBanks[0] = this._banks[address - 0x0fe0];
        }
        else if (address < 0x0ff0) {
            this._activeBanks[1] = this._banks[address - 0x0fe8];
        }
        else {
            this._activeBanks[2] = this._banks[address - 0x0ff0];
        }
    }
}
exports.default = CartridgeE0;
//# sourceMappingURL=CartridgeE0.js.map