"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const AbstractCartridge_1 = tslib_1.__importDefault(require("./AbstractCartridge"));
const CartridgeInfo_1 = tslib_1.__importDefault(require("./CartridgeInfo"));
class CartridgeF0 extends AbstractCartridge_1.default {
    constructor(buffer) {
        super();
        this._banks = new Array(16);
        this._bankIdx = 0;
        if (buffer.length !== 0x10000) {
            throw new Error(`buffer is not a 64k cartridge image: wrong length ${buffer.length}`);
        }
        for (let i = 0; i < 16; i++) {
            this._banks[i] = new Uint8Array(0x1000);
        }
        for (let i = 0; i < 0x1000; i++) {
            for (let j = 0; j < 16; j++) {
                this._banks[j][i] = buffer[j * 0x1000 + i];
            }
        }
        this.reset();
    }
    reset() {
        this._bankIdx = 0;
        this._currentBank = this._banks[this._bankIdx];
    }
    read(address) {
        address &= 0x0fff;
        this._handleBankswitch(address);
        return this._currentBank[address];
    }
    peek(address) {
        return this._currentBank[address & 0x0fff];
    }
    write(address, value) {
        address &= 0xfff;
        this._handleBankswitch(address);
        super.write(address, value);
    }
    getType() {
        return CartridgeInfo_1.default.CartridgeType.bankswitch_64k_F0;
    }
    _handleBankswitch(address) {
        if (address === 0x0ff0) {
            this._bankIdx = (this._bankIdx + 1) & 0x0f;
            this._currentBank = this._banks[this._bankIdx];
        }
    }
}
exports.default = CartridgeF0;
//# sourceMappingURL=CartridgeF0.js.map