"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const AbstractCartridge_1 = tslib_1.__importDefault(require("./AbstractCartridge"));
const CartridgeInfo_1 = tslib_1.__importDefault(require("./CartridgeInfo"));
class CartridgeF4 extends AbstractCartridge_1.default {
    constructor(buffer, _supportSC = true) {
        super();
        this._supportSC = _supportSC;
        this._bus = null;
        this._banks = new Array(8);
        this._ram = new Uint8Array(0x80);
        this._hasSC = false;
        if (buffer.length !== 0x8000) {
            throw new Error(`buffer is not a 32k cartridge image: wrong length ${buffer.length}`);
        }
        for (let i = 0; i < 8; i++) {
            this._banks[i] = new Uint8Array(0x1000);
        }
        for (let i = 0; i < 0x1000; i++) {
            for (let j = 0; j < 8; j++) {
                this._banks[j][i] = buffer[j * 0x1000 + i];
            }
        }
        this.reset();
    }
    reset() {
        this._bank = this._banks[0];
        this._hasSC = false;
    }
    getType() {
        return CartridgeInfo_1.default.CartridgeType.bankswitch_32k_F4;
    }
    randomize(rng) {
        for (let i = 0; i < this._ram.length; i++) {
            this._ram[i] = rng.int(0xff);
        }
    }
    setBus(bus) {
        this._bus = bus;
        return this;
    }
    read(address) {
        this._access(address & 0x0fff, this._bus.getLastDataBusValue());
        return this.peek(address);
    }
    peek(address) {
        address &= 0x0fff;
        if (this._hasSC && address >= 0x0080 && address < 0x0100) {
            return this._ram[address - 0x80];
        }
        else {
            return this._bank[address];
        }
    }
    write(address, value) {
        address &= 0x0fff;
        if (address < 0x80 && this._supportSC) {
            this._hasSC = true;
        }
        this._access(address, value);
    }
    _access(address, value) {
        if (address < 0x80 && this._hasSC) {
            this._ram[address] = value;
            return;
        }
        if (address >= 0x0ff4 && address <= 0x0ffb) {
            this._bank = this._banks[address - 0x0ff4];
        }
    }
}
exports.default = CartridgeF4;
//# sourceMappingURL=CartridgeF4.js.map