"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const AbstractCartridge_1 = tslib_1.__importDefault(require("./AbstractCartridge"));
const CartridgeInfo_1 = tslib_1.__importDefault(require("./CartridgeInfo"));
const cartridgeUtil = tslib_1.__importStar(require("./util"));
class CartridgeF8 extends AbstractCartridge_1.default {
    constructor(buffer, _supportSC = true) {
        super();
        this._supportSC = _supportSC;
        this._bank = null;
        this._bank0 = new Uint8Array(0x1000);
        this._bank1 = new Uint8Array(0x1000);
        this._hasSC = false;
        this._saraRAM = new Uint8Array(0x80);
        this._bus = null;
        if (buffer.length !== 0x2000) {
            throw new Error(`buffer is not an 8k cartridge image: wrong length ${buffer.length}`);
        }
        for (let i = 0; i < 0x1000; i++) {
            this._bank0[i] = buffer[i];
            this._bank1[i] = buffer[0x1000 + i];
        }
        this.reset();
    }
    static matchesBuffer(buffer) {
        const signatureCounts = cartridgeUtil.searchForSignatures(buffer, [[0x8d, 0xf9, 0x1f]]);
        return signatureCounts[0] >= 2;
    }
    reset() {
        this._bank = this._bank1;
        this._hasSC = false;
    }
    read(address) {
        this._access(address & 0x0fff, this._bus.getLastDataBusValue());
        return this.peek(address);
    }
    peek(address) {
        address &= 0x0fff;
        if (this._hasSC && address >= 0x0080 && address < 0x0100) {
            return this._saraRAM[address - 0x80];
        }
        return this._bank[address];
    }
    write(address, value) {
        address &= 0x0fff;
        if (address < 0x80 && this._supportSC) {
            this._hasSC = true;
        }
        this._access(address, value);
    }
    getType() {
        return CartridgeInfo_1.default.CartridgeType.bankswitch_8k_F8;
    }
    randomize(rng) {
        for (let i = 0; i < this._saraRAM.length; i++) {
            this._saraRAM[i] = rng.int(0xff);
        }
    }
    setBus(bus) {
        this._bus = bus;
        return this;
    }
    _access(address, value) {
        if (address < 0x80 && this._hasSC) {
            this._saraRAM[address] = value & 0xff;
            return;
        }
        switch (address) {
            case 0x0ff8:
                this._bank = this._bank0;
                break;
            case 0x0ff9:
                this._bank = this._bank1;
                break;
        }
    }
}
exports.default = CartridgeF8;
//# sourceMappingURL=CartridgeF8.js.map