"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const AbstractCartridge_1 = tslib_1.__importDefault(require("./AbstractCartridge"));
const CartridgeInfo_1 = tslib_1.__importDefault(require("./CartridgeInfo"));
class CartridgeFA extends AbstractCartridge_1.default {
    constructor(buffer) {
        super();
        this._bank0 = new Uint8Array(0x1000);
        this._bank1 = new Uint8Array(0x1000);
        this._bank2 = new Uint8Array(0x1000);
        this._ram = new Uint8Array(0x0100);
        if (buffer.length !== 0x3000) {
            throw new Error(`buffer is not a 12k cartridge image: wrong length ${buffer.length}`);
        }
        for (let i = 0; i < 0x1000; i++) {
            this._bank0[i] = buffer[i];
            this._bank1[i] = buffer[0x1000 + i];
            this._bank2[i] = buffer[0x2000 + i];
        }
        this.reset();
    }
    reset() {
        this._bank = this._bank0;
    }
    randomize(rng) {
        for (let i = 0; i < this._ram.length; i++) {
            this._ram[i] = rng.int(0xff);
        }
    }
    read(address) {
        this._handleBankswitch(address & 0x0fff);
        return this.peek(address);
    }
    peek(address) {
        address &= 0x0fff;
        if (address >= 0x0100 && address < 0x0200) {
            return this._ram[address & 0xff];
        }
        else {
            return this._bank[address];
        }
    }
    write(address, value) {
        address &= 0x0fff;
        this._handleBankswitch(address);
        if (address < 0x0100) {
            this._ram[address] = value & 0xff;
        }
        else {
            super.write(address, value);
        }
    }
    getType() {
        return CartridgeInfo_1.default.CartridgeType.bankswitch_12k_FA;
    }
    _handleBankswitch(address) {
        switch (address) {
            case 0x0ff8:
                this._bank = this._bank0;
                break;
            case 0x0ff9:
                this._bank = this._bank1;
                break;
            case 0x0ffa:
                this._bank = this._bank2;
                break;
        }
    }
}
exports.default = CartridgeFA;
//# sourceMappingURL=CartridgeFA.js.map