"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const AbstractCartridge_1 = tslib_1.__importDefault(require("./AbstractCartridge"));
const CartridgeInfo_1 = tslib_1.__importDefault(require("./CartridgeInfo"));
const cartridgeUtil = tslib_1.__importStar(require("./util"));
class CartridgeFE extends AbstractCartridge_1.default {
    constructor(buffer) {
        super();
        this._bank0 = new Uint8Array(0x1000);
        this._bank1 = new Uint8Array(0x1000);
        this._lastAccessWasFE = false;
        this._lastAddressBusValue = -1;
        if (buffer.length !== 0x2000) {
            throw new Error(`buffer is not an 8k cartridge image: wrong length ${buffer.length}`);
        }
        for (let i = 0; i < 0x1000; i++) {
            this._bank0[i] = buffer[i];
            this._bank1[i] = buffer[0x1000 + i];
        }
        this.reset();
    }
    static matchesBuffer(buffer) {
        const signatureCounts = cartridgeUtil.searchForSignatures(buffer, [
            [0x20, 0x00, 0xd0, 0xc6, 0xc5],
            [0x20, 0xc3, 0xf8, 0xa5, 0x82],
            [0xd0, 0xfb, 0x20, 0x73, 0xfe],
            [0x20, 0x00, 0xf0, 0x84, 0xd6]
        ]);
        for (let i = 0; i < signatureCounts.length; i++) {
            if (signatureCounts[i] > 0) {
                return true;
            }
        }
        return false;
    }
    reset() {
        this._bank = this._bank0;
        this._lastAccessWasFE = false;
        this._lastAddressBusValue = -1;
    }
    read(address) {
        return this._bank[address & 0x0fff];
    }
    write(address, value) {
        super.write(address, value);
    }
    setBus(bus) {
        this._bus = bus;
        this._bus.event.read.addHandler(CartridgeFE._onBusAccess, this);
        this._bus.event.write.addHandler(CartridgeFE._onBusAccess, this);
        return this;
    }
    getType() {
        return CartridgeInfo_1.default.CartridgeType.bankswitch_8k_FE;
    }
    static _onBusAccess(accessType, self) {
        const previousAddressBusValue = self._lastAddressBusValue;
        self._lastAddressBusValue = self._bus.getLastAddresBusValue() & 0x1fff;
        if (self._lastAddressBusValue === previousAddressBusValue) {
            return;
        }
        if (self._lastAccessWasFE) {
            const dataBusHiBits = self._bus.getLastDataBusValue() & 0xe0;
            self._bank =
                dataBusHiBits === 0
                    ? self._bank0
                    : (self._bus.getLastDataBusValue() & 0x20) > 0
                        ? self._bank0
                        : self._bank1;
        }
        self._lastAccessWasFE = self._lastAddressBusValue === 0x01fe;
    }
}
exports.default = CartridgeFE;
//# sourceMappingURL=CartridgeFE.js.map