"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const Cartridge2k_1 = tslib_1.__importDefault(require("./Cartridge2k"));
const Cartridge4k_1 = tslib_1.__importDefault(require("./Cartridge4k"));
const CartridgeCV_1 = tslib_1.__importDefault(require("./CartridgeCV"));
const CartridgeF8_1 = tslib_1.__importDefault(require("./CartridgeF8"));
const CartridgeF6_1 = tslib_1.__importDefault(require("./CartridgeF6"));
const CartridgeE0_1 = tslib_1.__importDefault(require("./CartridgeE0"));
const CartridgeFE_1 = tslib_1.__importDefault(require("./CartridgeFE"));
const Cartridge3F_1 = tslib_1.__importDefault(require("./Cartridge3F"));
const Cartridge3E_1 = tslib_1.__importDefault(require("./Cartridge3E"));
const CartridgeUA_1 = tslib_1.__importDefault(require("./CartridgeUA"));
const CartridgeFA_1 = tslib_1.__importDefault(require("./CartridgeFA"));
const CartridgeE7_1 = tslib_1.__importDefault(require("./CartridgeE7"));
const CartridgeF0_1 = tslib_1.__importDefault(require("./CartridgeF0"));
const CartridgeEF_1 = tslib_1.__importDefault(require("./CartridgeEF"));
const CartridgeF4_1 = tslib_1.__importDefault(require("./CartridgeF4"));
const CartridgeFA2_1 = tslib_1.__importDefault(require("./CartridgeFA2"));
const CartridgeSupercharger_1 = tslib_1.__importDefault(require("./CartridgeSupercharger"));
const CartridgeDPC_1 = tslib_1.__importDefault(require("./CartridgeDPC"));
const CartridgeDPCPlus_1 = tslib_1.__importDefault(require("./CartridgeDPCPlus"));
const CartridgeCDF_1 = tslib_1.__importDefault(require("./CartridgeCDF"));
const Cartridge0840_1 = tslib_1.__importDefault(require("./Cartridge0840"));
const CartridgePP_1 = tslib_1.__importDefault(require("./CartridgePP"));
const CartridgeInfo_1 = tslib_1.__importDefault(require("./CartridgeInfo"));
const CartridgeDetector_1 = tslib_1.__importDefault(require("./CartridgeDetector"));
class CartridgeFactory {
    createCartridge(buffer, cartridgeType) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const cartridge = this._createCartridge(buffer, cartridgeType);
            yield cartridge.init();
            return cartridge;
        });
    }
    _createCartridge(buffer, cartridgeType) {
        if (typeof cartridgeType === 'undefined') {
            const detector = new CartridgeDetector_1.default();
            cartridgeType = detector.detectCartridgeType(buffer);
        }
        switch (cartridgeType) {
            case CartridgeInfo_1.default.CartridgeType.vanilla_2k:
                return new Cartridge2k_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.vanilla_4k:
                return new Cartridge4k_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_2k_cv:
                return new CartridgeCV_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_8k_F8:
                return new CartridgeF8_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_8k_E0:
                return new CartridgeE0_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_8k_3F:
                return new Cartridge3F_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_8k_FE:
                return new CartridgeFE_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_8k_UA:
                return new CartridgeUA_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_8k_DPC:
                return new CartridgeDPC_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_8k_econobanking:
                return new Cartridge0840_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_8k_pp:
                return new CartridgePP_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_12k_FA:
                return new CartridgeFA_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_16k_F6:
                return new CartridgeF6_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_16k_E7:
                return new CartridgeE7_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_FA2:
                return new CartridgeFA2_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_32k_F4:
                return new CartridgeF4_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_64k_F0:
                return new CartridgeF0_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_64k_EF:
                return new CartridgeEF_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_3E:
                return new Cartridge3E_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_supercharger:
                return new CartridgeSupercharger_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_dpc_plus:
                return new CartridgeDPCPlus_1.default(buffer);
            case CartridgeInfo_1.default.CartridgeType.bankswitch_cdf:
                return new CartridgeCDF_1.default(buffer);
            default:
                throw new Error(`invalid or unsupported cartridge image`);
        }
    }
}
exports.default = CartridgeFactory;
//# sourceMappingURL=CartridgeFactory.js.map