"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Ball {
    constructor(_collisionMask, _flushLineCache) {
        this._collisionMask = _collisionMask;
        this._flushLineCache = _flushLineCache;
        this.color = 0xffffffff;
        this.collision = 0;
        this._enabledOld = false;
        this._enabledNew = false;
        this._enabled = false;
        this._hmmClocks = 0;
        this._counter = 0;
        this._moving = false;
        this._width = 1;
        this._effectiveWidth = 0;
        this._lastMovementTick = 0;
        this._rendering = false;
        this._renderCounter = -4;
        this._widths = new Uint8Array([1, 2, 4, 8]);
        this._delaying = false;
        this.reset();
    }
    reset() {
        this.color = 0xffffffff;
        this.collision = 0;
        this._width = 1;
        this._enabledOld = false;
        this._enabledNew = false;
        this._enabled = false;
        this._counter = 0;
        this._rendering = false;
        this._renderCounter = -4;
        this._moving = false;
        this._hmmClocks = 0;
        this._delaying = false;
        this._effectiveWidth = 0;
        this._lastMovementTick = 0;
    }
    enabl(value) {
        const enabledNewOldValue = this._enabledNew;
        this._enabledNew = (value & 2) > 0;
        if (enabledNewOldValue !== this._enabledNew && !this._delaying) {
            this._flushLineCache();
            this._updateEnabled();
        }
    }
    hmbl(value) {
        this._hmmClocks = (value >>> 4) ^ 0x8;
    }
    resbl(counter) {
        this._counter = counter;
        this._rendering = true;
        this._renderCounter = -4 + (counter - 157);
    }
    ctrlpf(value) {
        const width = this._widths[(value & 0x30) >>> 4];
        if (width !== this._width) {
            this._flushLineCache();
        }
        this._width = width;
    }
    vdelbl(value) {
        const oldDelaying = this._delaying;
        this._delaying = (value & 0x01) > 0;
        if (oldDelaying !== this._delaying) {
            this._flushLineCache();
            this._updateEnabled();
        }
    }
    startMovement() {
        this._moving = true;
    }
    movementTick(clock, apply) {
        this._lastMovementTick = this._counter;
        if (clock === this._hmmClocks) {
            this._moving = false;
        }
        if (this._moving && apply) {
            this.tick(false);
        }
        return this._moving;
    }
    tick(isReceivingHclock) {
        this.collision = this._rendering && this._renderCounter >= 0 && this._enabled ? 0 : this._collisionMask;
        const starfieldEffect = this._moving && isReceivingHclock;
        if (this._counter === 156) {
            const starfieldDelta = (this._counter - this._lastMovementTick + 160) % 4;
            this._rendering = true;
            this._renderCounter = -4;
            if (starfieldEffect && starfieldDelta === 3 && this._width < 4) {
                this._renderCounter++;
            }
            switch (starfieldDelta) {
                case 3:
                    this._effectiveWidth = this._width === 1 ? 2 : this._width;
                    break;
                case 2:
                    this._effectiveWidth = 0;
                    break;
                default:
                    this._effectiveWidth = this._width;
                    break;
            }
        }
        else if (this._rendering && ++this._renderCounter >= (starfieldEffect ? this._effectiveWidth : this._width)) {
            this._rendering = false;
        }
        if (++this._counter >= 160) {
            this._counter = 0;
        }
    }
    getPixel(colorIn) {
        return this.collision ? colorIn : this.color;
    }
    shuffleStatus() {
        const oldEnabledOld = this._enabledOld;
        this._enabledOld = this._enabledNew;
        if (this._delaying && this._enabledOld !== oldEnabledOld) {
            this._flushLineCache();
            this._updateEnabled();
        }
    }
    setColor(color) {
        if (color !== this.color && this._enabled) {
            this._flushLineCache();
        }
        this.color = color;
    }
    _updateEnabled() {
        this._enabled = this._delaying ? this._enabledOld : this._enabledNew;
    }
}
exports.default = Ball;
//# sourceMappingURL=Ball.js.map