"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
class DelayQueue {
    constructor(_length, size) {
        this._length = _length;
        this._nextIndex = 0;
        this._indices = new Uint8Array(0xff);
        this._queue = new Array(this._length);
        for (let i = 0; i < this._length; i++) {
            this._queue[i] = new QueueEntry(size);
        }
    }
    reset() {
        for (let i = 0; i < this._length; i++) {
            this._queue[i].nextIndex = 0;
        }
    }
    push(address, value, delay) {
        if (delay >= this._length) {
            throw new Error('delay exceeds queue length');
        }
        const currentIndex = this._indices[address];
        if (currentIndex < this._length) {
            this._queue[currentIndex].remove(address);
        }
        const index = (this._nextIndex + delay) % this._length;
        this._queue[index].push(address, value);
        this._indices[address] = index;
        return this;
    }
    execute(handler, scope) {
        const entry = this._queue[this._nextIndex];
        this._nextIndex = (this._nextIndex + 1) % this._length;
        for (let i = 0; i < entry.nextIndex; i++) {
            handler(entry.addresses[i], entry.values[i], scope);
            this._indices[entry.addresses[i]] = 0xff;
        }
        entry.nextIndex = 0;
    }
}
exports.default = DelayQueue;
class QueueEntry {
    constructor(size) {
        this.size = size;
        this.nextIndex = 0;
        this.addresses = new Uint8Array(size);
        this.values = new Uint8Array(size);
    }
    push(address, value) {
        if (this.nextIndex >= this.size) {
            throw new Error('delay queue overflow');
        }
        this.addresses[this.nextIndex] = address;
        this.values[this.nextIndex] = value;
        this.nextIndex++;
    }
    remove(address) {
        let i;
        for (i = 0; i < this.nextIndex; i++) {
            if (this.addresses[i] === address) {
                break;
            }
        }
        if (i < this.nextIndex) {
            this.addresses[i] = this.addresses[this.nextIndex - 1];
            this.values[i] = this.values[this.nextIndex - 1];
            this.nextIndex--;
        }
    }
}
//# sourceMappingURL=DelayQueue.js.map