"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const drawCounterDecodes_1 = require("./drawCounterDecodes");
class Missile {
    constructor(_collisionMask, _flushLineCache) {
        this._collisionMask = _collisionMask;
        this._flushLineCache = _flushLineCache;
        this.color = 0xffffffff;
        this.collision = 0;
        this._enabled = false;
        this._enam = false;
        this._resmp = -1;
        this._hmmClocks = 0;
        this._counter = 0;
        this._moving = false;
        this._width = 1;
        this._effectiveWidth = 0;
        this._lastMovementTick = 0;
        this._rendering = false;
        this._renderCounter = -4;
        this._widths = new Uint8Array([1, 2, 4, 8]);
        this.reset();
    }
    reset() {
        this.color = 0xffffffff;
        this._width = 1;
        this._enabled = false;
        this._counter = 0;
        this._rendering = false;
        this._renderCounter = -4;
        this._moving = false;
        this._hmmClocks = 0;
        this._decodes = drawCounterDecodes_1.decodesMissile[0];
        this._resmp = 0;
        this._enam = false;
        this._effectiveWidth = 0;
        this._lastMovementTick = 0;
    }
    enam(value) {
        const enam = (value & 2) > 0, enabled = enam && this._resmp === 0;
        if (enam !== this._enam || enabled !== this._enabled) {
            this._flushLineCache();
        }
        this._enam = enam;
        this._enabled = enabled;
    }
    hmm(value) {
        this._hmmClocks = (value >>> 4) ^ 0x8;
    }
    resm(counter, hblank) {
        this._counter = counter;
        if (this._rendering) {
            if (this._renderCounter < 0) {
                this._renderCounter = -4 + (counter - 157);
            }
            else {
                switch (this._width) {
                    case 8:
                        this._renderCounter = counter - 157 + (this._renderCounter >= 4 ? 4 : 0);
                        break;
                    case 4:
                        this._renderCounter = counter - 157;
                        break;
                    case 2:
                        if (hblank) {
                            this._rendering = this._renderCounter > 1;
                        }
                        else if (this._renderCounter === 0) {
                            this._renderCounter++;
                        }
                        break;
                    default:
                        if (hblank) {
                            this._rendering = this._renderCounter > 0;
                        }
                        break;
                }
            }
        }
    }
    resmp(value, player) {
        const resmp = value & 0x02;
        if (resmp === this._resmp) {
            return;
        }
        this._flushLineCache();
        this._resmp = resmp;
        if (resmp) {
            this._enabled = false;
        }
        else {
            this._enabled = this._enam;
            this._counter = player.getRespClock();
        }
    }
    nusiz(value) {
        this._width = this._widths[(value & 0x30) >>> 4];
        this._decodes = drawCounterDecodes_1.decodesMissile[value & 0x07];
        if (this._rendering && this._renderCounter >= this._width) {
            this._rendering = false;
        }
    }
    startMovement() {
        this._moving = true;
    }
    movementTick(clock, apply) {
        this._lastMovementTick = this._counter;
        if (clock === this._hmmClocks) {
            this._moving = false;
        }
        if (this._moving && apply) {
            this.tick(false);
        }
        return this._moving;
    }
    tick(isReceivingHclock) {
        this.collision = this._rendering && this._renderCounter >= 0 && this._enabled ? 0 : this._collisionMask;
        const starfieldEffect = this._moving && isReceivingHclock;
        if (this._decodes[this._counter] && !this._resmp) {
            const starfieldDelta = (this._counter - this._lastMovementTick + 160) % 4;
            this._rendering = true;
            this._renderCounter = -4;
            if (starfieldEffect && starfieldDelta === 3 && this._width < 4) {
                this._renderCounter++;
            }
            switch (starfieldDelta) {
                case 3:
                    this._effectiveWidth = this._width === 1 ? 2 : this._width;
                    break;
                case 2:
                    this._effectiveWidth = 0;
                    break;
                default:
                    this._effectiveWidth = this._width;
                    break;
            }
        }
        else if (this._rendering && ++this._renderCounter >= (starfieldEffect ? this._effectiveWidth : this._width)) {
            this._rendering = false;
        }
        if (++this._counter >= 160) {
            this._counter = 0;
        }
    }
    getPixel(colorIn) {
        return this.collision ? colorIn : this.color;
    }
    setColor(color) {
        if (color !== this.color && this._enabled) {
            this._flushLineCache();
        }
        this.color = color;
    }
}
exports.default = Missile;
//# sourceMappingURL=Missile.js.map