"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = exports.__init = void 0;
const tslib_1 = require("tslib");
const microevent_ts_1 = require("microevent.ts");
const PCMChannel_1 = tslib_1.__importDefault(require("./PCMChannel"));
const R_MAX = 30;
const R = 1;
const VOL_MAX = 0x1e;
const mixingTable = new Float32Array(VOL_MAX + 1);
var __init;
(function (__init) {
    for (let vol = 0; vol <= VOL_MAX; vol++) {
        mixingTable[vol] = vol / VOL_MAX * (R_MAX + R * VOL_MAX) / (R_MAX + R * vol);
    }
})(__init = exports.__init || (exports.__init = {}));
class PCMAudio {
    constructor(_config) {
        this._config = _config;
        this.newFrame = new microevent_ts_1.Event();
        this.togglePause = new microevent_ts_1.Event();
        this._currentOutputBuffer = null;
        this._bufferIndex = 0;
        this._sampleRate = 0;
        this._counter = 0;
        this._isActive = false;
        this._channel0 = new PCMChannel_1.default();
        this._channel1 = new PCMChannel_1.default();
        this._sampleRate = (this._config.tvMode === 0 ? 60 * 262 : 50 * 312) * 2;
        this._frameSize = (this._config.tvMode === 0 ? 262 : 312) * 4;
        this.reset();
    }
    getChannels() {
        return [
            {
                audv: value => this._channel0.audv(value),
                audc: value => this._channel0.audc(value),
                audf: value => this._channel0.audf(value),
                reset: () => this.reset(),
                setActive: active => this.setActive(active)
            },
            {
                audv: value => this._channel1.audv(value),
                audc: value => this._channel1.audc(value),
                audf: value => this._channel1.audf(value),
                reset: () => undefined,
                setActive: () => undefined
            }
        ];
    }
    reset() {
        this._bufferIndex = 0;
        this._counter = 0;
        this._channel0.reset();
        this._channel1.reset();
    }
    tick() {
        switch (this._counter) {
            case 9:
            case 81:
                this._channel0.phase0();
                this._channel1.phase0();
                break;
            case 37:
            case 149:
                this._currentOutputBuffer.getContent()[this._bufferIndex++] =
                    mixingTable[this._channel0.phase1() + this._channel1.phase1()];
                if (this._bufferIndex === this._currentOutputBuffer.getLength()) {
                    this._dispatchBuffer();
                }
                break;
        }
        if (++this._counter === 228) {
            this._counter = 0;
        }
    }
    isPaused() {
        return !this._isActive;
    }
    setActive(isActive) {
        if (isActive === this._isActive) {
            return;
        }
        this._isActive = isActive;
        this.togglePause.dispatch(!isActive);
    }
    getSampleRate() {
        return this._sampleRate;
    }
    getFrameSize() {
        return this._frameSize;
    }
    setFrameBufferFactory(factory) {
        this._bufferFactory = factory;
        if (!this._currentOutputBuffer && factory) {
            this._currentOutputBuffer = factory();
            this._bufferIndex = 0;
        }
    }
    _dispatchBuffer() {
        this.newFrame.dispatch(this._currentOutputBuffer);
        this._currentOutputBuffer = this._bufferFactory ? this._bufferFactory() : null;
        this._bufferIndex = 0;
    }
}
exports.default = PCMAudio;
//# sourceMappingURL=PCMAudio.js.map