"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
class PCMChannel {
    constructor() {
        this._audv = 0;
        this._audc = 0;
        this._audf = 0;
        this._clkEnable = false;
        this._noiseFeedback = false;
        this._noiseCounterBit4 = false;
        this._pulseCounterHold = false;
        this._divCounter = 0;
        this._noiseCounter = 0;
        this._pulseCounter = 0;
        this.reset();
    }
    reset() {
        this._audc = this._audf = this._audv = 0;
        this._clkEnable = false;
        this._noiseFeedback = false;
        this._noiseCounterBit4 = false;
        this._pulseCounterHold = false;
        this._divCounter = 0;
        this._noiseCounter = 0;
        this._pulseCounter = 0;
    }
    phase0() {
        if (this._clkEnable) {
            this._noiseCounterBit4 = !!(this._noiseCounter & 0x01);
            switch (this._audc & 0x03) {
                case 0x00:
                case 0x01:
                    this._pulseCounterHold = false;
                    break;
                case 0x02:
                    this._pulseCounterHold = (this._noiseCounter & 0x1e) !== 0x02;
                    break;
                case 0x03:
                    this._pulseCounterHold = !this._noiseCounterBit4;
                    break;
            }
            switch (this._audc & 0x03) {
                case 0x00:
                    this._noiseFeedback =
                        !!((this._pulseCounter ^ this._noiseCounter) & 0x01) ||
                            !(this._noiseCounter !== 0 || this._pulseCounter !== 0x0a) ||
                            !(this._audc & 0x0c);
                    break;
                default:
                    this._noiseFeedback =
                        !!((this._noiseCounter & 0x04 ? 1 : 0) ^ (this._noiseCounter & 0x01)) ||
                            this._noiseCounter === 0;
                    break;
            }
        }
        this._clkEnable = this._divCounter === this._audf;
        if (this._divCounter === this._audf || this._divCounter === 0x1f) {
            this._divCounter = 0;
        }
        else {
            this._divCounter++;
        }
    }
    phase1() {
        let pulseFeedback = false;
        if (this._clkEnable) {
            switch (this._audc >>> 2) {
                case 0x00:
                    pulseFeedback =
                        !!((this._pulseCounter & 0x02 ? 1 : 0) ^ (this._pulseCounter & 0x01)) &&
                            this._pulseCounter !== 0x0a &&
                            !!(this._audc & 0x03);
                    break;
                case 0x01:
                    pulseFeedback = !(this._pulseCounter & 0x08);
                    break;
                case 0x02:
                    pulseFeedback = !this._noiseCounterBit4;
                    break;
                case 0x03:
                    pulseFeedback = !(!!(this._pulseCounter & 0x02) || !(this._pulseCounter & 0x0e));
                    break;
            }
            this._noiseCounter >>>= 1;
            if (this._noiseFeedback) {
                this._noiseCounter |= 0x10;
            }
            if (!this._pulseCounterHold) {
                this._pulseCounter = ~(this._pulseCounter >>> 1) & 0x07;
                if (pulseFeedback) {
                    this._pulseCounter |= 0x08;
                }
            }
        }
        return (this._pulseCounter & 0x01) * this._audv;
    }
    audc(value) {
        this._audc = value & 0x0f;
    }
    audf(value) {
        this._audf = value & 0x1f;
    }
    audv(value) {
        this._audv = value & 0x0f;
    }
}
exports.default = PCMChannel;
//# sourceMappingURL=PCMChannel.js.map