"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const C = 68e-9, RPOT = 1e6, R0 = 1.8e3, U = 5, LINES_FULL = 380;
class PaddleReader {
    constructor(clockFreq, _paddle) {
        this._paddle = _paddle;
        this._uThresh = 0;
        this._u = 0;
        this._dumped = false;
        this._value = 0.5;
        this._timestamp = 0;
        this._cpuTimeProvider = null;
        this._uThresh = U * (1 - Math.exp(-LINES_FULL * 228 / clockFreq / (RPOT + R0) / C));
        this._paddle.valueChanged.addHandler((value) => {
            this._updateValue();
            this._value = value;
        });
        this.reset();
    }
    setCpuTimeProvider(provider) {
        this._cpuTimeProvider = provider;
        this._timestamp = this._cpuTimeProvider();
    }
    reset() {
        this._u = 0;
        this._value = this._paddle.getValue();
        this._dumped = false;
        this._timestamp = this._cpuTimeProvider ? this._cpuTimeProvider() : 0;
    }
    vblank(value) {
        const oldValue = this._dumped;
        if (value & 0x80) {
            this._dumped = true;
            this._u = 0;
        }
        else if (oldValue) {
            this._dumped = false;
            this._timestamp = this._cpuTimeProvider();
        }
    }
    inpt() {
        this._updateValue();
        const state = this._dumped ? false : this._u >= this._uThresh;
        return state ? 0x80 : 0;
    }
    _updateValue() {
        if (this._dumped) {
            return;
        }
        const timestamp = this._cpuTimeProvider();
        this._u =
            U * (1 - (1 - this._u / U) * Math.exp(-(timestamp - this._timestamp) / (this._value * RPOT + R0) / C));
        this._timestamp = timestamp;
    }
}
exports.default = PaddleReader;
//# sourceMappingURL=PaddleReader.js.map