"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const drawCounterDecodes_1 = require("./drawCounterDecodes");
class Player {
    constructor(_collisionMask, _flushLineCache) {
        this._collisionMask = _collisionMask;
        this._flushLineCache = _flushLineCache;
        this.color = 0xffffffff;
        this.collision = 0;
        this._hmmClocks = 0;
        this._counter = 0;
        this._moving = false;
        this._divider = 1;
        this._dividerPending = 1;
        this._dividerChangeCounter = -1;
        this._sampleCounter = 0;
        this._rendering = false;
        this._renderCounter = -5;
        this._renderCounterTripPoint = 0;
        this._patternNew = 0;
        this._patternOld = 0;
        this._pattern = 0;
        this._reflected = false;
        this._delaying = false;
        this.reset();
    }
    reset() {
        this.color = 0xffffffff;
        this.collision = 0;
        this._hmmClocks = 0;
        this._counter = 0;
        this._moving = false;
        this._rendering = false;
        this._renderCounter = -5;
        this._decodes = drawCounterDecodes_1.decodesPlayer[0];
        this._patternNew = 0;
        this._patternOld = 0;
        this._pattern = 0;
        this._reflected = false;
        this._delaying = false;
        this._sampleCounter = 0;
        this._dividerPending = 0;
        this._dividerChangeCounter = -1;
        this._setDivider(1);
    }
    grp(pattern) {
        if (pattern === this._patternNew) {
            return;
        }
        this._patternNew = pattern;
        if (!this._delaying) {
            this._flushLineCache();
            this._updatePattern();
        }
    }
    hmp(value) {
        this._hmmClocks = (value >>> 4) ^ 0x8;
    }
    nusiz(value, hblank) {
        const masked = value & 0x07;
        switch (masked) {
            case 5:
                this._dividerPending = 2;
                break;
            case 7:
                this._dividerPending = 4;
                break;
            default:
                this._dividerPending = 1;
        }
        const oldDecodes = this._decodes;
        this._decodes = drawCounterDecodes_1.decodesPlayer[masked];
        if (this._decodes !== oldDecodes &&
            this._rendering &&
            this._renderCounter - -5 < 2 &&
            !this._decodes[(this._counter - this._renderCounter + -5 + 159) % 160]) {
            this._rendering = false;
        }
        if (this._dividerPending === this._divider) {
            return;
        }
        if (!this._rendering) {
            this._setDivider(this._dividerPending);
            return;
        }
        const delta = this._renderCounter - -5;
        switch ((this._divider << 4) | this._dividerPending) {
            case 0x12:
            case 0x14:
                if (hblank) {
                    if (delta < 4) {
                        this._setDivider(this._dividerPending);
                    }
                    else {
                        this._dividerChangeCounter = delta < 5 ? 1 : 0;
                    }
                }
                else {
                    if (delta < 3) {
                        this._setDivider(this._dividerPending);
                    }
                    else {
                        this._dividerChangeCounter = 1;
                    }
                }
                break;
            case 0x21:
            case 0x41:
                if (delta < (hblank ? 4 : 3)) {
                    this._setDivider(this._dividerPending);
                }
                else if (delta < (hblank ? 6 : 5)) {
                    this._setDivider(this._dividerPending);
                    this._renderCounter--;
                }
                else {
                    this._dividerChangeCounter = hblank ? 0 : 1;
                }
                break;
            case 0x42:
            case 0x24:
                if (this._renderCounter < 1 || (hblank && this._renderCounter % this._divider === 1)) {
                    this._setDivider(this._dividerPending);
                }
                else {
                    this._dividerChangeCounter = this._divider - (this._renderCounter - 1) % this._divider;
                }
                break;
            default:
                throw new Error('cannot happen');
        }
    }
    resp(counter) {
        this._counter = counter;
        if (this._rendering && this._renderCounter - -5 < 4) {
            this._renderCounter = -5 + (counter - 157);
        }
    }
    refp(value) {
        const oldReflected = this._reflected;
        this._reflected = (value & 0x08) > 0;
        if (this._reflected !== oldReflected) {
            this._flushLineCache();
            this._updatePattern();
        }
    }
    vdelp(value) {
        const oldDelaying = this._delaying;
        this._delaying = (value & 0x01) > 0;
        if (this._delaying !== oldDelaying) {
            this._flushLineCache();
            this._updatePattern();
        }
    }
    startMovement() {
        this._moving = true;
    }
    movementTick(clock, apply) {
        if (clock === this._hmmClocks) {
            this._moving = false;
        }
        if (this._moving && apply) {
            this.tick();
        }
        return this._moving;
    }
    tick() {
        this.collision =
            this._rendering &&
                this._renderCounter >= this._renderCounterTripPoint &&
                this._pattern & (1 << this._sampleCounter)
                ? 0
                : this._collisionMask;
        if (this._decodes[this._counter]) {
            this._rendering = true;
            this._renderCounter = -5;
            this._sampleCounter = 0;
        }
        else if (this._rendering) {
            this._renderCounter++;
            switch (this._divider) {
                case 1:
                    if (this._renderCounter > 0) {
                        this._sampleCounter++;
                    }
                    if (this._renderCounter >= 0 &&
                        this._dividerChangeCounter >= 0 &&
                        this._dividerChangeCounter-- === 0) {
                        this._setDivider(this._dividerPending);
                    }
                    break;
                default:
                    if (this._renderCounter > 1 && (this._renderCounter - 1) % this._divider === 0) {
                        this._sampleCounter++;
                    }
                    if (this._renderCounter > 0 &&
                        this._dividerChangeCounter >= 0 &&
                        this._dividerChangeCounter-- === 0) {
                        this._setDivider(this._dividerPending);
                    }
                    break;
            }
            if (this._sampleCounter > 7) {
                this._rendering = false;
            }
        }
        if (++this._counter >= 160) {
            this._counter = 0;
        }
    }
    getPixel(colorIn) {
        return this.collision ? colorIn : this.color;
    }
    shufflePatterns() {
        const oldPatternOld = this._patternOld;
        this._patternOld = this._patternNew;
        if (this._delaying && oldPatternOld !== this._patternOld) {
            this._flushLineCache();
            this._updatePattern();
        }
    }
    getRespClock() {
        switch (this._divider) {
            case 1:
                return (this._counter - 5 + 160) % 160;
            case 2:
                return (this._counter - 9 + 160) % 160;
            case 4:
                return (this._counter - 12 + 160) % 160;
            default:
                throw new Error(`cannot happen: invalid divider ${this._divider}`);
        }
    }
    setColor(color) {
        if (color !== this.color && this._pattern) {
            this._flushLineCache();
        }
        this.color = color;
    }
    _updatePattern() {
        this._pattern = this._delaying ? this._patternOld : this._patternNew;
        if (!this._reflected) {
            this._pattern =
                ((this._pattern & 0x01) << 7) |
                    ((this._pattern & 0x02) << 5) |
                    ((this._pattern & 0x04) << 3) |
                    ((this._pattern & 0x08) << 1) |
                    ((this._pattern & 0x10) >>> 1) |
                    ((this._pattern & 0x20) >>> 3) |
                    ((this._pattern & 0x40) >>> 5) |
                    ((this._pattern & 0x80) >>> 7);
        }
    }
    _setDivider(divider) {
        this._divider = divider;
        this._renderCounterTripPoint = divider === 1 ? 0 : 1;
    }
}
exports.default = Player;
//# sourceMappingURL=Player.js.map