"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
class Playfield {
    constructor(_collisionMask, _flushLineCache) {
        this._collisionMask = _collisionMask;
        this._flushLineCache = _flushLineCache;
        this.collision = 0;
        this._colorLeft = 0;
        this._colorRight = 0;
        this._color = 0;
        this._colorP0 = 0;
        this._colorP1 = 0;
        this._colorMode = 0;
        this._pattern = 0;
        this._refp = false;
        this._reflected = false;
        this._pf0 = 0;
        this._pf1 = 0;
        this._pf2 = 0;
        this._x = 0;
        this.reset();
    }
    reset() {
        this._pattern = 0;
        this._reflected = false;
        this._refp = false;
        this._pf0 = 0;
        this._pf1 = 0;
        this._pf2 = 0;
        this._color = 0;
        this._colorP0 = 0;
        this._colorP1 = 0;
        this._colorMode = 0;
        this._applyColors();
    }
    pf0(value) {
        if (this._pf0 === value >>> 4) {
            return;
        }
        this._flushLineCache();
        this._pf0 = value >>> 4;
        this._pattern = (this._pattern & 0x000ffff0) | this._pf0;
    }
    pf1(value) {
        if (this._pf1 === value) {
            return;
        }
        this._flushLineCache();
        this._pf1 = value;
        this._pattern =
            (this._pattern & 0x000ff00f) |
                ((value & 0x80) >>> 3) |
                ((value & 0x40) >>> 1) |
                ((value & 0x20) << 1) |
                ((value & 0x10) << 3) |
                ((value & 0x08) << 5) |
                ((value & 0x04) << 7) |
                ((value & 0x02) << 9) |
                ((value & 0x01) << 11);
    }
    pf2(value) {
        if (this._pf2 === value) {
            return;
        }
        this._flushLineCache();
        this._pf2 = value;
        this._pattern = (this._pattern & 0x00000fff) | ((value & 0xff) << 12);
    }
    ctrlpf(value) {
        const reflected = (value & 0x01) > 0, colorMode = (value & 0x06) === 0x02 ? 1 : 0;
        if (reflected === this._reflected && colorMode === this._colorMode) {
            return;
        }
        this._flushLineCache();
        this._reflected = reflected;
        this._colorMode = colorMode;
        this._applyColors();
    }
    setColor(color) {
        if (color !== this._color && this._colorMode === 0) {
            this._flushLineCache();
        }
        this._color = color;
        this._applyColors();
    }
    setColorP0(color) {
        if (color !== this._colorP0 && this._colorMode === 1) {
            this._flushLineCache();
        }
        this._colorP0 = color;
        this._applyColors();
    }
    setColorP1(color) {
        if (color !== this._colorP1 && this._colorMode === 1) {
            this._flushLineCache();
        }
        this._colorP1 = color;
        this._applyColors();
    }
    tick(x) {
        this._x = x;
        if (x === 80 || x === 0) {
            this._refp = this._reflected;
        }
        if (x & 0x03) {
            return;
        }
        let currentPixel;
        if (this._pattern === 0) {
            currentPixel = 0;
        }
        else if (x < 80) {
            currentPixel = this._pattern & (1 << (x >>> 2));
        }
        else if (this._refp) {
            currentPixel = this._pattern & (1 << (39 - (x >>> 2)));
        }
        else {
            currentPixel = this._pattern & (1 << ((x >>> 2) - 20));
        }
        this.collision = currentPixel ? 0 : this._collisionMask;
    }
    getPixel(colorIn) {
        if (!this.collision) {
            return this._x < 80 ? this._colorLeft : this._colorRight;
        }
        return colorIn;
    }
    _applyColors() {
        switch (this._colorMode) {
            case 0:
                this._colorLeft = this._colorRight = this._color;
                break;
            case 1:
                this._colorLeft = this._colorP0;
                this._colorRight = this._colorP1;
                break;
        }
    }
}
exports.default = Playfield;
//# sourceMappingURL=Playfield.js.map