"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const Config_1 = tslib_1.__importDefault(require("../Config"));
const AudioOutputBuffer_1 = tslib_1.__importDefault(require("../../../tools/AudioOutputBuffer"));
const base64_1 = require("../../../tools/base64");
const FREQUENCY_DIVISIORS = base64_1.decode('AQEPAQEBAQEBAQEBAwMDAQ==');
const POLY0 = new Int8Array([1]);
const POLY1 = new Int8Array([1, 1]);
const POLY2 = new Int8Array([16, 15]);
const POLY4 = base64_1.decode('AQICAQEBBAM=');
const POLY5 = base64_1.decode('AQIBAQICBQQCAQMBAQEBBA==');
const POLY9 = base64_1.decode('AQQBAwIEAQIDAgEBAQEBAQIEAgEEAQECAgEDAgEDAQEBBAEBAQECAQECBgECAgECAQIBAQIBBg' +
    'IBAgIBAQEBAgICAgcCAwICAQEBAwIBAQIBAQcBAQMBAQIDAwEBAQICAQECAgQDBQEDAQEFAgEB' +
    'AQIBAgEDAQIFAQECAQEBBQEBAQEBAQEBBgEBAQIBAQEBBAIBAQMBAwYDAgMBAQIBAgQBAQEDAQ' +
    'EBAQMBAgEEAgIDBAEBBAECAQICAgEBBAMBBAQJBQQBBQMBAQMCAgIBBQECAQEBAgMBAgEBAwQC' +
    'BQICAQIDAQEBAQECAQMDAwIBAgEBAQEBAwMBAgIDAQMBCA==');
const POLY68 = base64_1.decode('BQYEBQoFAwcECgYDBgQJBg==');
const POLY465 = base64_1.decode('AgMCAQQBBgoCBAIBAQQFCQMDBAEBAQgFBQUEAQEBCAQCCAMDAQEHBAIHBQEDAQcEAQQIAgEDBA' +
    'cBAwcDAgEGBgICBAUDAgYGAQMDAgUDBwMEAwICAgUJAwEFAwECAgsFAQUDAQECDAUBAgUCAQEM' +
    'BgECBQECAQoGAwICBAECBgo=');
const POLYS = [
    POLY0,
    POLY4,
    POLY4,
    POLY465,
    POLY1,
    POLY1,
    POLY2,
    POLY5,
    POLY9,
    POLY5,
    POLY2,
    POLY0,
    POLY1,
    POLY1,
    POLY2,
    POLY68
];
class ToneGenerator {
    constructor(_config) {
        this._config = _config;
    }
    setConfig(config) {
        this._config = config;
    }
    getKey(tone, frequency) {
        if (POLYS[tone] === POLY1 && FREQUENCY_DIVISIORS[tone] * (frequency + 1) === 1) {
            return 0;
        }
        return (tone << 5) | frequency;
    }
    getBuffer(key) {
        const tone = (key >>> 5) & 0x0f, frequency = key & 0x1f;
        const poly = POLYS[tone];
        let length = 0;
        for (let i = 0; i < poly.length; i++) {
            length += poly[i];
        }
        length = length * FREQUENCY_DIVISIORS[tone] * (frequency + 1);
        const content = new Float32Array(length);
        const sampleRate = Config_1.default.getClockHz(this._config) / 114;
        let f = 0;
        let count = 0;
        let offset = 0;
        let state = true;
        for (let i = 0; i < length; i++) {
            f++;
            if (f === FREQUENCY_DIVISIORS[tone] * (frequency + 1)) {
                f = 0;
                count++;
                if (count === poly[offset]) {
                    offset++;
                    count = 0;
                    if (poly.length === offset) {
                        offset = 0;
                    }
                }
                state = !(offset & 0x01);
            }
            content[i] = state ? 1 : -1;
        }
        return new AudioOutputBuffer_1.default(content, sampleRate);
    }
}
exports.default = ToneGenerator;
//# sourceMappingURL=ToneGenerator.js.map