"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const microevent_ts_1 = require("microevent.ts");
const ToneGenerator_1 = tslib_1.__importDefault(require("./ToneGenerator"));
class WaveformAudio {
    constructor(_config) {
        this._config = _config;
        this.bufferChanged = new microevent_ts_1.Event();
        this.volumeChanged = new microevent_ts_1.Event();
        this.stop = new microevent_ts_1.Event();
        this._volume = -1;
        this._tone = -1;
        this._frequency = -1;
        this._active = false;
        this._toneGenerator = null;
        this._toneGenerator = new ToneGenerator_1.default(this._config);
        this.reset();
    }
    reset() {
        this._volume = -1;
        this._tone = -1;
        this._frequency = -1;
    }
    audc(value) {
        value &= 0x0f;
        if (value === this._tone) {
            return;
        }
        this._tone = value;
        this._dispatchBufferChanged();
    }
    audf(value) {
        value &= 0x1f;
        if (value === this._frequency) {
            return;
        }
        this._frequency = value;
        this._dispatchBufferChanged();
    }
    audv(value) {
        value &= 0x0f;
        if (value === this._volume) {
            return;
        }
        this._volume = value / 15;
        this.volumeChanged.dispatch(this._volume);
    }
    setActive(active) {
        this._active = active;
        if (active) {
            this._dispatchBufferChanged();
        }
        else {
            this.stop.dispatch(undefined);
        }
    }
    getVolume() {
        return this._volume >= 0 ? this._volume : 0;
    }
    getBuffer(key) {
        return this._toneGenerator.getBuffer(key);
    }
    _getKey() {
        return this._toneGenerator.getKey(this._tone, this._frequency);
    }
    _dispatchBufferChanged() {
        if (this._active && this.bufferChanged.hasHandlers) {
            this.bufferChanged.dispatch(this._getKey());
        }
    }
}
exports.default = WaveformAudio;
//# sourceMappingURL=WaveformAudio.js.map