"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const microevent_ts_1 = require("microevent.ts");
class ClockProbe {
    constructor(_scheduler) {
        this._scheduler = _scheduler;
        this.frequencyUpdate = new microevent_ts_1.Event();
        this._counter = 0;
        this._frequency = 0;
    }
    attach(clock) {
        if (this._clock) {
            this.detach();
        }
        this._clock = clock;
        clock.addHandler(this._clockHandler, this);
        return this;
    }
    start() {
        if (this._measurementTask) {
            return this;
        }
        this._timestamp = Date.now();
        this._counter = 0;
        this._measurementTask = this._scheduler.start(this._updateMeasurement, this);
        return this;
    }
    detach() {
        if (!this._clock) {
            return this;
        }
        this._clock.removeHandler(this._clockHandler, this);
        this._clock = undefined;
        return this;
    }
    stop() {
        if (!this._measurementTask) {
            return this;
        }
        this._measurementTask.stop();
        this._measurementTask = undefined;
        return this;
    }
    getFrequency() {
        return this._frequency;
    }
    _updateMeasurement(probe) {
        const timestamp = Date.now();
        probe._frequency = probe._counter / (timestamp - probe._timestamp) * 1000;
        probe._counter = 0;
        probe._timestamp = timestamp;
        probe.frequencyUpdate.dispatch(probe._frequency);
    }
    _clockHandler(clocks, ctx) {
        ctx._counter += clocks;
    }
}
exports.default = ClockProbe;
//# sourceMappingURL=ClockProbe.js.map