"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const microevent_ts_1 = require("microevent.ts");
class RingBuffer {
    constructor(_capacity) {
        this._capacity = _capacity;
        this.evict = new microevent_ts_1.Event();
        this._size = 0;
        this._index = 0;
        this._buffer = new Array(this._capacity);
        for (let i = 0; i < this._capacity; i++) {
            this._buffer[i] = null;
        }
    }
    size() {
        return this._size;
    }
    pop() {
        if (this._size === 0) {
            return undefined;
        }
        const item = this._buffer[this._index];
        this._buffer[this._index] = null;
        this._index = (this._index + 1) % this._capacity;
        this._size--;
        return item;
    }
    push(item) {
        if (this._size === this._capacity) {
            this.evict.dispatch(this.pop());
        }
        this._buffer[(this._index + this._size++) % this._capacity] = item;
        return this;
    }
    forEach(fn) {
        for (let i = 0; i < this._size; i++) {
            fn(this._buffer[(this._index + i) % this._capacity]);
        }
        return this;
    }
    clear() {
        for (let i = 0; i < this._capacity; i++) {
            this._buffer[i] = null;
        }
        this._size = 0;
        this._index = 0;
        return this;
    }
    capacity() {
        return this._capacity;
    }
}
exports.default = RingBuffer;
//# sourceMappingURL=RingBuffer.js.map