"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const microevent_ts_1 = require("microevent.ts");
const PoolMember_1 = tslib_1.__importDefault(require("./PoolMember"));
class Pool {
    constructor(_factory) {
        this._factory = _factory;
        this.event = {
            release: new microevent_ts_1.Event(),
            dispose: new microevent_ts_1.Event(),
        };
        this._pool = [];
        this._poolSize = 0;
    }
    get() {
        let member;
        if (this._poolSize === 0) {
            const newItem = this._factory();
            member = new PoolMember_1.default(newItem, (victim) => this._releaseMember(victim), (victim) => this._disposeMember(victim));
        }
        else {
            member = this._pool[--this._poolSize];
            member._isAvailable = false;
        }
        return member;
    }
    _releaseMember(victim) {
        if (victim._isAvailable) {
            throw new Error('Trying to release an already released pool member');
        }
        if (victim._isDisposed) {
            throw new Error('Trying to release an already disposed pool member');
        }
        const position = this._poolSize++;
        this._pool[position] = victim;
        victim._isAvailable = true;
        victim._poolPosition = position;
        this.event.release.dispatch(victim.get());
    }
    _disposeMember(victim) {
        if (victim._isDisposed) {
            throw new Error('Trying to dispose of an already disposed pool member');
        }
        if (victim._isAvailable) {
            if (this._poolSize > 1) {
                this._pool[victim._poolPosition] = this._pool[this._poolSize - 1];
            }
            this._poolSize--;
        }
        victim._isDisposed = true;
        this.event.dispose.dispatch(victim.get());
    }
}
exports.default = Pool;
//# sourceMappingURL=Pool.js.map