"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const PeriodicScheduler_1 = tslib_1.__importDefault(require("./PeriodicScheduler"));
const ImmedateScheduler_1 = tslib_1.__importDefault(require("./ImmedateScheduler"));
const BusyWait_1 = tslib_1.__importDefault(require("./limiting/BusyWait"));
const ConstantCycles_1 = tslib_1.__importDefault(require("./limiting/ConstantCycles"));
const ConstantTimeslice_1 = tslib_1.__importDefault(require("./limiting/ConstantTimeslice"));
class Factory {
    createPeriodicScheduler(period) {
        return new PeriodicScheduler_1.default(period);
    }
    createImmediateScheduler() {
        return new ImmedateScheduler_1.default();
    }
    createLimitingScheduler(strategy = 1) {
        switch (strategy) {
            case 0:
                return new BusyWait_1.default();
            case 1:
                return new ConstantCycles_1.default();
            case 2:
                return new ConstantTimeslice_1.default();
            default:
                throw new Error('invalud limiting scheduling strategy');
        }
    }
    getLimitingSchedulingStrategies() {
        return [
            0,
            1,
            2
        ];
    }
    describeLimitingSchedulingStrategy(strategy) {
        switch (strategy) {
            case 0:
                return 'Busy wait, constant timeslice length';
            case 1:
                return 'Constant cycle count';
            case 2:
                return 'Constant timeslice length';
            default:
                throw new Error('invalid limiting scheduling strategy');
        }
    }
}
exports.default = Factory;
//# sourceMappingURL=Factory.js.map