"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const getTimestamp_1 = tslib_1.__importDefault(require("../getTimestamp"));
const setImmediate_1 = require("../setImmediate");
const CORRECTION_THESHOLD = 3, MAX_ACCUMULATED_DELTA = 100;
class ConstantCyclesScheduler {
    start(worker, context) {
        let terminate = false, targetSleepInterval = -1, lastYieldTimestamp = 0, accumulatedDelta = 0;
        function handler() {
            if (terminate) {
                return;
            }
            const timestamp0 = getTimestamp_1.default(), targetDuration = worker(context) || 0, timestamp1 = getTimestamp_1.default();
            let delay = targetDuration - timestamp1 + timestamp0;
            if (targetSleepInterval >= 0) {
                accumulatedDelta += targetSleepInterval - timestamp0 + lastYieldTimestamp;
            }
            if (accumulatedDelta > MAX_ACCUMULATED_DELTA) {
                accumulatedDelta = MAX_ACCUMULATED_DELTA;
            }
            else if (accumulatedDelta < -MAX_ACCUMULATED_DELTA) {
                accumulatedDelta = -MAX_ACCUMULATED_DELTA;
            }
            if (Math.abs(accumulatedDelta) > CORRECTION_THESHOLD) {
                delay += accumulatedDelta;
                accumulatedDelta = 0;
            }
            if (delay < 0) {
                delay = 0;
                accumulatedDelta = delay;
            }
            if (delay > 0) {
                setTimeout(handler, Math.round(delay));
            }
            else {
                setImmediate_1.setImmediate(handler);
            }
            targetSleepInterval = delay;
            lastYieldTimestamp = getTimestamp_1.default();
        }
        setImmediate_1.setImmediate(handler);
        return {
            stop: () => (terminate = true)
        };
    }
}
exports.default = ConstantCyclesScheduler;
//# sourceMappingURL=ConstantCycles.js.map