"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
class ArrayBufferSurface {
    constructor() {
        this._height = 0;
        this._width = 0;
        this._buffer = null;
    }
    static createFromArrayBuffer(width, height, buffer) {
        return new ArrayBufferSurface().replaceUnderlyingBuffer(width, height, buffer);
    }
    replaceUnderlyingBuffer(width, height, buffer) {
        if (width * height * 4 !== buffer.byteLength) {
            throw new Error('surface size mismatch');
        }
        this._width = width;
        this._height = height;
        this._underlyingBuffer = buffer;
        this._buffer = new Uint32Array(this._underlyingBuffer);
        return this;
    }
    getUnderlyingBuffer() {
        return this._underlyingBuffer;
    }
    resetUnderlyingBuffer() {
        this._width = this._height = 0;
        this._underlyingBuffer = this._buffer = null;
        return this;
    }
    getWidth() {
        return this._width;
    }
    getHeight() {
        return this._height;
    }
    getBuffer() {
        return this._buffer;
    }
    getByteOrder() {
        return 0;
    }
    fill(value) {
        for (let i = 0; i < this._buffer.length; i++) {
            this._buffer[i] = value;
        }
        return this;
    }
}
exports.default = ArrayBufferSurface;
//# sourceMappingURL=ArrayBufferSurface.js.map