"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const screenfull_1 = tslib_1.__importDefault(require("screenfull"));
const browser_1 = require("../../tools/browser");
const noFullscrenApi = browser_1.isIOS;
const screenfull = screenfull_1.default;
class FullscreenVideoDriver {
    constructor(_videoDriver, _zIndex = 100000, _fullscreenClass = 'stellerator-fullscreen') {
        this._videoDriver = _videoDriver;
        this._zIndex = _zIndex;
        this._fullscreenClass = _fullscreenClass;
        this._resizeListener = () => {
            if (this._resizeHandle) {
                return;
            }
            this._resizeHandle = setTimeout(() => {
                this._resizeHandle = null;
                this._adjustSizeForFullscreen();
            }, 100);
        };
        this._resizeHandle = null;
        this._changeListener = this._onChange.bind(this);
        this._engaged = false;
    }
    engage() {
        if (this._engaged) {
            return;
        }
        this._engaged = true;
        if (noFullscrenApi || !screenfull) {
            this._adjustSizeForFullscreen();
            window.addEventListener('resize', this._resizeListener);
            this._engaged = true;
        }
        else {
            screenfull.on('change', this._changeListener);
            screenfull.request(this._videoDriver.getCanvas());
        }
    }
    disengage() {
        if (!this._engaged) {
            return;
        }
        if (noFullscrenApi || !screenfull) {
            this._resetSize();
            window.removeEventListener('resize', this._resizeListener);
            this._engaged = false;
        }
        else {
            screenfull.exit();
        }
    }
    toggle() {
        if (this._engaged) {
            this.disengage();
        }
        else {
            this.engage();
        }
    }
    isEngaged() {
        return this._engaged;
    }
    _onChange() {
        if (!screenfull) {
            return;
        }
        if (screenfull.isFullscreen) {
            window.addEventListener('resize', this._resizeListener);
            this._adjustSizeForFullscreen();
        }
        else {
            this._resetSize();
            window.removeEventListener('resize', this._resizeListener);
            screenfull.off('change', this._changeListener);
            this._engaged = false;
        }
    }
    _resetSize() {
        const element = this._videoDriver.getCanvas();
        if (this._resizeHandle) {
            clearTimeout(this._resizeHandle);
            this._resizeHandle = null;
        }
        element.style.width = '';
        element.style.height = '';
        element.style.maxWidth = '';
        element.style.maxHeight = '';
        if (noFullscrenApi) {
            element.style.position = '';
            element.style.top = '';
            element.style.left = '';
            element.style.zIndex = '';
        }
        document.body.classList.remove(this._fullscreenClass);
        setTimeout(() => this._videoDriver.resize(), 0);
    }
    _adjustSizeForFullscreen() {
        const element = this._videoDriver.getCanvas();
        this._videoDriver.resize(window.innerWidth, window.innerHeight);
        element.style.width = window.innerWidth + 'px';
        element.style.height = window.innerHeight + 'px';
        element.style.maxWidth = window.innerWidth + 'px';
        element.style.maxHeight = window.innerHeight + 'px';
        if (noFullscrenApi) {
            element.style.position = 'fixed';
            element.style.top = '0';
            element.style.left = '0';
            element.style.zIndex = '' + this._zIndex;
        }
        document.body.classList.add(this._fullscreenClass);
    }
}
exports.default = FullscreenVideoDriver;
//# sourceMappingURL=FullscreenVideo.js.map