"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const microevent_ts_1 = require("microevent.ts");
const Pool_1 = tslib_1.__importDefault(require("../../tools/pool/Pool"));
const InducedPool_1 = tslib_1.__importDefault(require("../../tools/pool/InducedPool"));
const AudioOutputBuffer_1 = tslib_1.__importDefault(require("../../tools/AudioOutputBuffer"));
class PCMAudioEndpoint {
    constructor(_output) {
        this._output = _output;
        this.newFrame = new microevent_ts_1.Event();
        this.togglePause = new microevent_ts_1.Event();
        this._audioBufferPool = new Pool_1.default(() => new AudioOutputBuffer_1.default(new Float32Array(this.getFrameSize()), this.getSampleRate()));
        this._audioBufferMap = new WeakMap();
        this._pcmDataPool = new InducedPool_1.default((buffer) => buffer.getContent(), (value, target) => value.get().replaceUnderlyingBuffer(target));
        this._output.newFrame.addHandler((buffer) => this.newFrame.dispatch(this._pcmDataPool.get(this._audioBufferMap.get(buffer))));
        this._output.togglePause.addHandler((paused) => this.togglePause.dispatch(paused));
        this._output.setFrameBufferFactory(() => {
            const wrappedBuffer = this._audioBufferPool.get();
            if (!this._audioBufferMap.has(wrappedBuffer.get())) {
                this._audioBufferMap.set(wrappedBuffer.get(), wrappedBuffer);
            }
            return wrappedBuffer.get();
        });
    }
    getSampleRate() {
        return this._output.getSampleRate();
    }
    getFrameSize() {
        return this._output.getFrameSize();
    }
    isPaused() {
        return this._output.isPaused();
    }
}
exports.default = PCMAudioEndpoint;
//# sourceMappingURL=PCMAudioEndpoint.js.map