"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const microevent_ts_1 = require("microevent.ts");
const Pool_1 = tslib_1.__importDefault(require("../../tools/pool/Pool"));
const ArrayBufferSurface_1 = tslib_1.__importDefault(require("../../video/surface/ArrayBufferSurface"));
class VideoEndpoint {
    constructor(_video) {
        this._video = _video;
        this.newFrame = new microevent_ts_1.Event();
        this._poolMembers = new WeakMap();
        this._surfaces = new WeakMap();
        this._pool = new Pool_1.default(() => new ImageData(this._video.getWidth(), this._video.getHeight()));
        this._video.setSurfaceFactory(() => {
            const poolMember = this._pool.get(), imageData = poolMember.get();
            if (!this._surfaces.has(imageData)) {
                const newSurface = ArrayBufferSurface_1.default.createFromArrayBuffer(imageData.width, imageData.height, imageData.data.buffer);
                this._surfaces.set(imageData, newSurface.fill(0xff000000));
            }
            const surface = this._surfaces.get(imageData);
            this._poolMembers.set(surface, poolMember);
            return surface;
        });
        this._video.newFrame.addHandler(imageData => this.newFrame.dispatch(this._poolMembers.get(imageData)));
    }
    getWidth() {
        return this._video.getWidth();
    }
    getHeight() {
        return this._video.getHeight();
    }
}
exports.default = VideoEndpoint;
//# sourceMappingURL=VideoEndpoint.js.map