"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
class WaveformChannel {
    constructor(_cache) {
        this._cache = _cache;
        this._context = null;
        this._source = null;
        this._gain = null;
        this._audio = null;
        this._volume = 0;
        this._masterVolume = 1;
    }
    init(context, target) {
        this._context = context;
        this._gain = this._context.createGain();
        this._gain.connect(target);
    }
    bind(target) {
        if (this._audio) {
            return;
        }
        this._audio = target;
        this._volume = this._audio.getVolume();
        this._updateGain();
        this._audio.volumeChanged.addHandler(WaveformChannel._onVolumeChanged, this);
        this._audio.bufferChanged.addHandler(WaveformChannel._onBufferChanged, this);
        this._audio.stop.addHandler(WaveformChannel._onStop, this);
    }
    unbind() {
        if (!this._audio) {
            return;
        }
        this._audio.volumeChanged.removeHandler(WaveformChannel._onVolumeChanged, this);
        this._audio.bufferChanged.removeHandler(WaveformChannel._onBufferChanged, this);
        this._audio.stop.removeHandler(WaveformChannel._onStop, this);
        if (this._source) {
            this._source.stop();
            this._source = null;
        }
        this._audio = null;
    }
    setMasterVolume(volume) {
        this._masterVolume = volume;
        this._updateGain();
    }
    static _onVolumeChanged(volume, self) {
        self._volume = volume;
        self._updateGain();
    }
    static _onBufferChanged(key, self) {
        if (!self._cache.has(key)) {
            const sampleBuffer = self._audio.getBuffer(key), audioBuffer = self._context.createBuffer(1, sampleBuffer.getLength(), sampleBuffer.getSampleRate());
            audioBuffer.getChannelData(0).set(sampleBuffer.getContent());
            self._cache.set(key, audioBuffer);
        }
        const buffer = self._cache.get(key), source = self._context.createBufferSource();
        if (self._source) {
            self._source.stop();
        }
        source.loop = true;
        source.buffer = buffer;
        source.connect(self._gain);
        source.start();
        self._source = source;
    }
    static _onStop(payload, self) {
        if (self._source) {
            self._source.stop();
            self._source = null;
        }
    }
    _updateGain() {
        this._gain.gain.value = this._volume * this._masterVolume;
    }
}
exports.default = WaveformChannel;
//# sourceMappingURL=WaveformChannel.js.map