"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.detect = void 0;
function framebufferSupportTextureType(gl, type) {
    const texture = gl.createTexture();
    const framebuffer = gl.createFramebuffer();
    gl.activeTexture(gl.TEXTURE0);
    gl.bindTexture(gl.TEXTURE_2D, texture);
    gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, 64, 64, 0, gl.RGBA, type, null);
    gl.bindFramebuffer(gl.FRAMEBUFFER, framebuffer);
    gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, texture, 0);
    const fbStatus = gl.checkFramebufferStatus(gl.FRAMEBUFFER);
    gl.deleteFramebuffer(framebuffer);
    gl.deleteTexture(texture);
    return fbStatus === gl.FRAMEBUFFER_COMPLETE;
}
function detectFloatTextureSupport(gl) {
    gl.getExtension('WEBGL_color_buffer_float');
    if (!gl.getExtension('OES_texture_float'))
        return false;
    return framebufferSupportTextureType(gl, gl.FLOAT);
}
function detectHalfFloatTextureSupport(gl) {
    gl.getExtension('EXT_color_buffer_half_float');
    const extHalfFLoat = gl.getExtension('OES_texture_half_float');
    if (!extHalfFLoat) {
        return false;
    }
    return framebufferSupportTextureType(gl, extHalfFLoat.HALF_FLOAT_OES);
}
function shaderSupportsPrecision(gl, shaderType, precisionType) {
    const format = gl.getShaderPrecisionFormat(shaderType, precisionType);
    return !!format && format.precision > 0;
}
function detect(gl = null) {
    if (!gl) {
        const canvas = document.createElement('canvas');
        canvas.width = 64;
        canvas.height = 64;
        gl = canvas.getContext('webgl') || canvas.getContext('experimental-webgl');
    }
    if (!gl)
        return null;
    return {
        floatTextures: detectFloatTextureSupport(gl),
        halfFloatTextures: detectHalfFloatTextureSupport(gl),
        highpInFsh: shaderSupportsPrecision(gl, gl.FRAGMENT_SHADER, gl.HIGH_FLOAT),
        highpInVsh: shaderSupportsPrecision(gl, gl.VERTEX_SHADER, gl.HIGH_FLOAT)
    };
}
exports.detect = detect;
//# sourceMappingURL=Capabilities.js.map