import { EventInterface as Event } from 'microevent.ts';
import CartridgeInfo from '../../../machine/stella/cartridge/CartridgeInfo';
import ControlPanel from './ControlPanel';
export declare class Stellerator {
    constructor(canvasElt: HTMLCanvasElement | null, workerUrl: string, config?: Partial<Stellerator.Config>);
    setGamma(gamma: number): this;
    getGamma(): number;
    setScalingMode(scalingMode: Stellerator.ScalingMode): this;
    getScalingMode(): Stellerator.ScalingMode;
    setTvEmulation(tvEmulation: Stellerator.TvEmulation): this;
    getTvEmulation(): Stellerator.TvEmulation;
    setPhosphorLevel(phosphorLevel: number): this;
    getPhosphorLevel(): number;
    setScanlineLevel(scanlineLevel: number): this;
    getScanlineLevel(): number;
    toggleFullscreen(fullscreen?: boolean): this;
    isFullscreen(): boolean;
    setVolume(volume: number): this;
    audioEnabled(): boolean;
    getVolume(): number;
    resize(): this;
    getState(): Stellerator.State;
    getControlPanel(): ControlPanel;
    setCanvas(canvas: HTMLCanvasElement): this;
    releaseCanvas(): this;
    start(cartridge: ArrayLike<number> | string, tvMode: Stellerator.TvMode, config?: Partial<Stellerator.CartridgeConfig>): Promise<Stellerator.State>;
    run(cartridge: ArrayLike<number> | string, tvMode: Stellerator.TvMode, config?: Partial<Stellerator.CartridgeConfig>): Promise<Stellerator.State>;
    pause(): Promise<Stellerator.State>;
    resume(): Promise<Stellerator.State>;
    stop(): Promise<Stellerator.State>;
    reset(): Promise<Stellerator.State>;
    lastError(): Error;
    asyncIOSend(message: ArrayLike<number>): this;
    private _convertTvMode;
    private _createDrivers;
    private _removeVideoDriver;
    private _createVideoDriver;
    private _removeTouchDriver;
    private _createTouchDriver;
    private _mapState;
    private _pauseHandler;
    frequencyUpdate: Event<number>;
    asyncIOMessage: Event<ArrayLike<number>>;
    stateChange: Event<Stellerator.State>;
    private _canvasElt;
    private _config;
    private _emulationService;
    private _serviceInitialized;
    private _videoDriver;
    private _fullscreenVideo;
    private _audioDriver;
    private _keyboardIO;
    private _asyncIO;
    private _touchIO;
    private _paddle;
    private _gamepad;
    private _controlPanel;
    private _state;
    private _driverManager;
    private _mutex;
}
export declare namespace Stellerator {
    interface Config {
        gamma: number;
        scalingMode: ScalingMode;
        tvEmulation: TvEmulation;
        phosphorLevel: number;
        scanlineLevel: number;
        audio: boolean;
        volume: number;
        enableKeyboard: boolean;
        enableTouch: boolean;
        touchLeftHanded: boolean;
        touchJoystickSensitivity: number;
        keyboardTarget: HTMLElement | HTMLDocument;
        fullscreenViaKeyboard: boolean;
        pauseViaKeyboard: boolean;
        resetViaKeyboard: boolean;
        pauseViaTouch: boolean;
        fullscreenViaTouch: boolean;
        paddleViaMouse: boolean;
        enableGamepad: boolean;
    }
    enum TvMode {
        ntsc = "ntsc",
        pal = "pal",
        secam = "secam"
    }
    enum TvEmulation {
        composite = "composite",
        svideo = "svideo",
        none = "none"
    }
    enum ScalingMode {
        qis = "qis",
        bilinear = "bilinear",
        none = "none"
    }
    interface CartridgeConfig {
        cartridgeType: CartridgeInfo.CartridgeType;
        randomSeed: number;
        emulatePaddles: boolean;
        frameStart: number;
        cpuAccuracy: CpuAccuracy;
        asyncIO: boolean;
    }
    const CartridgeType: typeof CartridgeInfo.CartridgeType;
    const describeCartridgeType: (cartridgeType: CartridgeInfo.CartridgeType) => string;
    const allCartridgeTypes: () => Array<CartridgeInfo.CartridgeType>;
    enum State {
        running = "running",
        paused = "paused",
        stopped = "stopped",
        error = "error"
    }
    enum CpuAccuracy {
        cycle = "cycle",
        instruction = "instruction"
    }
}
export { Stellerator as default };
