"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const microevent_ts_1 = require("microevent.ts");
class SwitchProxy {
    constructor() {
        this.stateChange = new microevent_ts_1.Event();
        this._state = false;
        this._boundSwitch = null;
    }
    bind(swtch) {
        this.unbind();
        this._boundSwitch = swtch;
        this._boundSwitch.toggle(this._state);
        this._boundSwitch.stateChanged.addHandler(SwitchProxy._onBoundStateChange, this);
        this._setState(this._boundSwitch.read());
    }
    unbind() {
        if (!this._boundSwitch) {
            return;
        }
        this._boundSwitch.stateChanged.removeHandler(SwitchProxy._onBoundStateChange, this);
        this._boundSwitch = null;
    }
    toggle(state) {
        if (this._boundSwitch) {
            this._boundSwitch.toggle(state);
        }
        else {
            this._setState(state);
        }
        return this;
    }
    read() {
        return this._state;
    }
    static _onBoundStateChange(newState, self) {
        self._setState(newState);
    }
    _setState(newState) {
        if (newState !== this._state) {
            this._state = newState;
            this.stateChange.dispatch(this._state);
        }
    }
}
exports.default = SwitchProxy;
//# sourceMappingURL=SwitchProxy.js.map