"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const WebAudio_1 = tslib_1.__importDefault(require("../../driver/WebAudio"));
class WebAudioDriver {
    constructor(_fragmentSize) {
        this._fragmentSize = _fragmentSize;
        this._pcmAudio = false;
        this._volume = 1;
    }
    init() { }
    bind(pcmAudio, channels) {
        if (this._channels) {
            return;
        }
        this._channels = [...channels];
        if (!this._driver || this._pcmAudio !== pcmAudio) {
            if (this._driver) {
                this._driver.close();
            }
            this._driver = pcmAudio
                ? new WebAudio_1.default(0, this._channels.length, this._fragmentSize)
                : new WebAudio_1.default(this._channels.length, 0, this._fragmentSize);
            this._driver.init();
        }
        if (pcmAudio) {
            this._driver.bind([], this._channels);
        }
        else {
            this._driver.bind(this._channels, []);
        }
        for (let i = 0; i < this._channels.length; i++) {
            this._driver.setMasterVolume(i, this._volume);
        }
        this._pcmAudio = pcmAudio;
    }
    unbind() {
        if (!this._channels) {
            return;
        }
        this._driver.unbind();
        this._channels = null;
    }
    setMasterVolume(volume) {
        this._volume = volume;
        if (this._channels) {
            for (let i = 0; i < this._channels.length; i++) {
                this._driver.setMasterVolume(i, this._volume);
            }
        }
    }
    getMasterVolume() {
        return this._volume;
    }
    pause() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this._driver) {
                yield this._driver.pause();
            }
        });
    }
    resume() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this._driver) {
                yield this._driver.resume();
            }
        });
    }
}
exports.default = WebAudioDriver;
//# sourceMappingURL=WebAudio.js.map