"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const microevent_ts_1 = require("microevent.ts");
class DoubleTapDetector {
    constructor(_maxTapLength = 500, _timeout = 200) {
        this._maxTapLength = _maxTapLength;
        this._timeout = _timeout;
        this.trigger = new microevent_ts_1.Event();
        this._dispatch = false;
        this._touching = false;
        this._lastTouchEligible = false;
        this._lastTouchStart = 0;
        this._lastTouchEnd = 0;
    }
    startTouch() {
        this._lastTouchStart = Date.now();
        if (this._touching) {
            return;
        }
        this._touching = true;
        this._dispatch = this._lastTouchEligible && this._lastTouchStart - this._lastTouchEnd < this._timeout;
    }
    endTouch() {
        this._lastTouchEnd = Date.now();
        if (this._dispatch) {
            this._dispatch = false;
            this.trigger.dispatch(undefined);
        }
        if (!this._touching) {
            return;
        }
        this._touching = false;
        this._lastTouchEligible = this._lastTouchStart - this._lastTouchEnd < this._maxTapLength;
    }
    cancelTouch() {
        this.endTouch();
        this._lastTouchEligible = false;
    }
    isDispatching() {
        return this._dispatch;
    }
}
exports.default = DoubleTapDetector;
//# sourceMappingURL=DoubleTapDetector.js.map