"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const VideoEndpoint_1 = tslib_1.__importDefault(require("../../../driver/VideoEndpoint"));
const PCMAudioEndpoint_1 = tslib_1.__importDefault(require("../../../driver/PCMAudioEndpoint"));
class EmulationContext {
    constructor(_board, _asyncIO) {
        this._board = _board;
        this._asyncIO = _asyncIO;
        this._videoEndpoint = null;
        this._audioEndpoint = null;
    }
    getConfig() {
        return this._board.getConfig();
    }
    getVideo() {
        if (!this._videoEndpoint) {
            this._videoEndpoint = new VideoEndpoint_1.default(this._board.getVideoOutput());
        }
        return this._videoEndpoint;
    }
    getJoystick(i) {
        switch (i) {
            case 0:
                return this._board.getJoystick0();
            case 1:
                return this._board.getJoystick1();
            default:
                throw new Error(`invalid joystick index ${i}`);
        }
    }
    getControlPanel() {
        return this._board.getControlPanel();
    }
    getPaddle(i) {
        if (i >= 0 && i < 4) {
            return this._board.getPaddle(i);
        }
        else {
            throw new Error(`invalid paddle index ${i}`);
        }
    }
    getWaveformChannels() {
        return this._board.getWaveformChannels();
    }
    getPCMChannel() {
        if (!this._audioEndpoint) {
            this._audioEndpoint = new PCMAudioEndpoint_1.default(this._board.getPCMChannel());
        }
        return this._audioEndpoint;
    }
    getRawVideo() {
        if (this._videoEndpoint) {
            throw new Error(`video endpoint already initialized; raw video unavailable`);
        }
        return this._board.getVideoOutput();
    }
    getAsyncIO() {
        return this._asyncIO;
    }
}
exports.default = EmulationContext;
//# sourceMappingURL=EmulationContext.js.map