"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const messages_1 = require("./messages");
class ControlDriver {
    constructor(_rpc) {
        this._rpc = _rpc;
        this._active = false;
        this._emulationContext = null;
    }
    init() {
        this._rpc.registerSignalHandler(messages_1.SIGNAL_TYPE.controlStateUpdate, this._onControlStateUpdate.bind(this));
    }
    bind(emulationContext) {
        if (this._active) {
            return;
        }
        this._active = true;
        this._emulationContext = emulationContext;
    }
    unbind() {
        if (!this._active) {
            return;
        }
        this._active = false;
        this._emulationContext = null;
    }
    _onControlStateUpdate(controlState) {
        if (!this._active) {
            return;
        }
        for (let i = 0; i < 2; i++) {
            this._applyJoystickState(controlState.joystickState[i], this._emulationContext.getJoystick(i));
        }
        for (let i = 0; i < 4; i++) {
            this._applyPaddleState(controlState.paddleState[i], this._emulationContext.getPaddle(i));
        }
        this._applyControlPanelState(controlState.controlPanelState, this._emulationContext.getControlPanel());
    }
    _applyJoystickState(state, joystick) {
        joystick.getUp().toggle(state.up);
        joystick.getDown().toggle(state.down);
        joystick.getLeft().toggle(state.left);
        joystick.getRight().toggle(state.right);
        joystick.getFire().toggle(state.fire);
    }
    _applyPaddleState(state, paddle) {
        paddle.setValue(state.value);
        paddle.getFire().toggle(state.fire);
    }
    _applyControlPanelState(state, panel) {
        panel.getDifficultySwitchP0().toggle(state.difficulty0);
        panel.getDifficultySwitchP1().toggle(state.difficulty1);
        panel.getResetButton().toggle(state.reset);
        panel.getColorSwitch().toggle(state.color);
        panel.getSelectSwitch().toggle(state.select);
    }
}
exports.default = ControlDriver;
//# sourceMappingURL=ControlDriver.js.map