"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const DigitalJoystick_1 = tslib_1.__importDefault(require("../../../../machine/io/DigitalJoystick"));
const ControlPanel_1 = tslib_1.__importDefault(require("../../../../machine/stella/ControlPanel"));
const Paddle_1 = tslib_1.__importDefault(require("../../../../machine/io/Paddle"));
const messages_1 = require("./messages");
class ControlProxy {
    constructor(_rpc) {
        this._rpc = _rpc;
        this._joysticks = new Array(2);
        this._paddles = new Array(4);
        this._controlPanel = new ControlPanel_1.default();
        for (let i = 0; i < 2; i++) {
            this._joysticks[i] = new DigitalJoystick_1.default();
        }
        for (let i = 0; i < 4; i++) {
            this._paddles[i] = new Paddle_1.default();
        }
    }
    sendUpdate() {
        this._rpc.signal(messages_1.SIGNAL_TYPE.controlStateUpdate, {
            joystickState: this._joysticks.map(ControlProxy._joystickState),
            paddleState: this._paddles.map(ControlProxy._paddleState),
            controlPanelState: ControlProxy._controlPanelState(this._controlPanel)
        });
    }
    getJoystick(i) {
        if (i < 0 || i > 1) {
            throw new Error(`invalid joystick index ${i}`);
        }
        return this._joysticks[i];
    }
    getControlPanel() {
        return this._controlPanel;
    }
    getPaddle(i) {
        if (i < 0 || i > 3) {
            throw new Error(`invalid paddle index ${i}`);
        }
        return this._paddles[i];
    }
    static _joystickState(joystick) {
        return {
            up: joystick.getUp().read(),
            down: joystick.getDown().read(),
            left: joystick.getLeft().read(),
            right: joystick.getRight().read(),
            fire: joystick.getFire().read()
        };
    }
    static _paddleState(paddle) {
        return {
            value: paddle.getValue(),
            fire: paddle.getFire().read()
        };
    }
    static _controlPanelState(controlPanel) {
        return {
            difficulty0: controlPanel.getDifficultySwitchP0().read(),
            difficulty1: controlPanel.getDifficultySwitchP1().read(),
            select: controlPanel.getSelectSwitch().read(),
            reset: controlPanel.getResetButton().read(),
            color: controlPanel.getColorSwitch().read()
        };
    }
}
exports.default = ControlProxy;
//# sourceMappingURL=ControlProxy.js.map