"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const messages_1 = require("./messages");
class PCMAudioDriver {
    constructor(index, _rpc) {
        this._rpc = _rpc;
        this._enabled = false;
        this._sampleRate = 0;
        this._frameSize = 0;
        this._pendingFrames = new Map();
        this._nextId = 0;
        this._paused = false;
        this._signalNewFrame = '';
        this._signalTogglePause = '';
        this._rpc
            .registerRpcHandler(messages_1.RPC_TYPE.getPCMAudioParameters(index), this._onGetPCMAudioParameters.bind(this))
            .registerSignalHandler(messages_1.SIGNAL_TYPE.pcmAudioReturnFrame(index), this._onReturnFrame.bind(this));
        this._signalNewFrame = messages_1.SIGNAL_TYPE.pcmAudioNewFrame(index);
        this._signalTogglePause = messages_1.SIGNAL_TYPE.pcmAudioTogglePause(index);
    }
    bind(endpoint) {
        if (this._enabled) {
            this.unbind();
        }
        this._endpoint = endpoint;
        this._endpoint.newFrame.addHandler(PCMAudioDriver._onNewFrame, this);
        this._endpoint.togglePause.addHandler(PCMAudioDriver._onTogglePause, this);
        this._sampleRate = this._endpoint.getSampleRate();
        this._frameSize = this._endpoint.getFrameSize();
        this._enabled = true;
    }
    unbind() {
        if (!this._enabled) {
            return;
        }
        this._endpoint.newFrame.removeHandler(PCMAudioDriver._onNewFrame, this);
        this._endpoint.togglePause.removeHandler(PCMAudioDriver._onTogglePause, this);
        this._endpoint = null;
        this._pendingFrames.clear();
        this._sampleRate = this._frameSize = 0;
        this._enabled = false;
    }
    static _onNewFrame(frame, self) {
        if (!self._enabled) {
            frame.dispose();
            return;
        }
        const id = self._nextId++, data = frame.get();
        self._pendingFrames.set(id, frame);
        self._rpc.signal(self._signalNewFrame, {
            id,
            buffer: data.buffer
        }, [data.buffer]);
    }
    static _onTogglePause(paused, self) {
        self._paused = paused;
        self._rpc.signal(self._signalTogglePause, {
            paused
        });
    }
    _onGetPCMAudioParameters() {
        return {
            sampleRate: this._sampleRate,
            frameSize: this._frameSize,
            paused: this._paused
        };
    }
    _onReturnFrame(msg) {
        if (!this._enabled || !this._pendingFrames.has(msg.id)) {
            return;
        }
        const frame = this._pendingFrames.get(msg.id);
        this._pendingFrames.delete(msg.id);
        frame.adopt(new Float32Array(msg.buffer));
        frame.release();
    }
}
exports.default = PCMAudioDriver;
//# sourceMappingURL=PCMAudioDriver.js.map