"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const microevent_ts_1 = require("microevent.ts");
const Pool_1 = tslib_1.__importDefault(require("../../../../tools/pool/Pool"));
const messages_1 = require("./messages");
class PCMAudioProxy {
    constructor(_index, _rpc) {
        this._index = _index;
        this._rpc = _rpc;
        this.newFrame = new microevent_ts_1.Event();
        this.togglePause = new microevent_ts_1.Event();
        this._sampleRate = 0;
        this._frameSize = 0;
        this._paused = false;
        this._framePool = new Pool_1.default(() => null);
        this._frameMap = new WeakMap();
        this._enabled = false;
        this._signalReturnFrame = '';
        this._framePool.event.release.addHandler(PCMAudioProxy._onReleaseFragment, this);
        this._signalReturnFrame = messages_1.SIGNAL_TYPE.pcmAudioReturnFrame(this._index);
    }
    init() {
        this._rpc
            .registerSignalHandler(messages_1.SIGNAL_TYPE.pcmAudioNewFrame(this._index), this._onNewFrame.bind(this))
            .registerSignalHandler(messages_1.SIGNAL_TYPE.pcmAudioTogglePause(this._index), this._onTogglePause.bind(this));
        return this;
    }
    start() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this._enabled) {
                return;
            }
            const params = yield this._rpc.rpc(messages_1.RPC_TYPE.getPCMAudioParameters(this._index));
            this._sampleRate = params.sampleRate;
            this._frameSize = params.frameSize;
            this._paused = params.paused;
            this._enabled = true;
        });
    }
    stop() {
        this._enabled = false;
    }
    isPaused() {
        return this._paused;
    }
    getSampleRate() {
        return this._sampleRate;
    }
    getFrameSize() {
        return this._frameSize;
    }
    static _onReleaseFragment(frame, self) {
        if (!self._frameMap.has(frame)) {
            return;
        }
        self._rpc.signal(self._signalReturnFrame, {
            id: self._frameMap.get(frame),
            buffer: frame.buffer
        }, [frame.buffer]);
    }
    _onNewFrame(msg) {
        if (!this._enabled) {
            return;
        }
        const frame = this._framePool.get(), data = new Float32Array(msg.buffer);
        frame.adopt(data);
        this._frameMap.set(data, msg.id);
        this.newFrame.dispatch(frame);
    }
    _onTogglePause(msg) {
        if (msg.paused === this._paused) {
            return;
        }
        this._paused = msg.paused;
        this.togglePause.dispatch(this._paused);
    }
}
exports.default = PCMAudioProxy;
//# sourceMappingURL=PCMAudioProxy.js.map