"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const microevent_ts_1 = require("microevent.ts");
const Pool_1 = tslib_1.__importDefault(require("../../../../tools/pool/Pool"));
const messages_1 = require("./messages");
class VideoProxy {
    constructor(_rpc) {
        this._rpc = _rpc;
        this.newFrame = new microevent_ts_1.Event();
        this._framePool = new Pool_1.default(() => null);
        this._frameMap = null;
        this._active = false;
        this._width = 0;
        this._height = 0;
        this._framePool.event.release.addHandler(VideoProxy._onDisposeFrame, this);
    }
    init() {
        this._rpc.registerSignalHandler(messages_1.SIGNAL_TYPE.videoNewFrame, this._onNewFrame.bind(this));
    }
    start() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this._active) {
                this.stop();
            }
            const videoParameters = yield this._rpc.rpc(messages_1.RPC_TYPE.getVideoParameters);
            this._active = true;
            this._width = videoParameters.width;
            this._height = videoParameters.height;
            this._frameMap = new WeakMap();
        });
    }
    stop() {
        this._active = false;
    }
    getWidth() {
        return this._width;
    }
    getHeight() {
        return this._height;
    }
    static _onDisposeFrame(imageData, self) {
        if (!self._active)
            return;
        if (!self._frameMap.has(imageData)) {
            console.warn('unknown imageData returned to proxy');
            return;
        }
        const id = self._frameMap.get(imageData);
        self._frameMap.delete(imageData);
        self._rpc.signal(messages_1.SIGNAL_TYPE.videoReturnSurface, {
            id,
            buffer: imageData.data.buffer
        }, [imageData.data.buffer]);
    }
    _onNewFrame(message) {
        if (!this._active) {
            console.warn('video proxy deactivated: ignoring frame');
            return;
        }
        if (this._width !== message.width || this._height !== message.height) {
            console.warn(`surface dimensions do not match; ignoring frame`);
            return;
        }
        const frame = this._framePool.get();
        const imageData = new ImageData(new Uint8ClampedArray(message.buffer), message.width, message.height);
        frame.adopt(imageData);
        this._frameMap.set(imageData, message.id);
        this.newFrame.dispatch(frame);
    }
}
exports.default = VideoProxy;
//# sourceMappingURL=VideoProxy.js.map