"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const messages_1 = require("./messages");
class WaveformAudioDriver {
    constructor(_index, _rpc) {
        this._index = _index;
        this._rpc = _rpc;
        this._audio = null;
        this._rpc.registerRpcHandler(messages_1.RPC_TYPE.getWaveformAudioParameters(this._index), this._onGetWaveformAudioParameters.bind(this));
    }
    bind(audio) {
        if (this._audio) {
            return;
        }
        this._audio = audio;
        this._audio.bufferChanged.addHandler(WaveformAudioDriver._onBufferChanged, this);
        this._audio.volumeChanged.addHandler(WaveformAudioDriver._onVolumeChanged, this);
        this._audio.stop.addHandler(WaveformAudioDriver._onStop, this);
    }
    unbind() {
        if (!this._audio) {
            return;
        }
        this._audio.bufferChanged.removeHandler(WaveformAudioDriver._onBufferChanged, this);
        this._audio.volumeChanged.removeHandler(WaveformAudioDriver._onVolumeChanged, this);
        this._audio.stop.removeHandler(WaveformAudioDriver._onStop, this);
        this._audio = null;
    }
    static _onBufferChanged(key, self) {
        self._rpc.signal(messages_1.SIGNAL_TYPE.waveformAudioBufferChange, {
            index: self._index,
            key
        });
    }
    static _onVolumeChanged(value, self) {
        self._rpc.signal(messages_1.SIGNAL_TYPE.waveformAudioVolumeChange, {
            index: self._index,
            value
        });
    }
    static _onStop(value, self) {
        self._rpc.signal(messages_1.SIGNAL_TYPE.audioStop, self._index);
    }
    _onGetWaveformAudioParameters() {
        return { volume: this._audio.getVolume() };
    }
}
exports.default = WaveformAudioDriver;
//# sourceMappingURL=WaveformAudioDriver.js.map