"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = void 0;
const tslib_1 = require("tslib");
const microevent_ts_1 = require("microevent.ts");
const ToneGenerator_1 = tslib_1.__importDefault(require("../../../../machine/stella/tia/ToneGenerator"));
const messages_1 = require("./messages");
class WaveformAudioProxy {
    constructor(_index, _rpc) {
        this._index = _index;
        this._rpc = _rpc;
        this.bufferChanged = new microevent_ts_1.Event();
        this.volumeChanged = new microevent_ts_1.Event();
        this.stop = new microevent_ts_1.Event();
        this._toneGenerator = new ToneGenerator_1.default();
        this._volume = 0;
    }
    init() {
        this._rpc
            .registerSignalHandler(messages_1.SIGNAL_TYPE.waveformAudioBufferChange, this._onBufferChangeSignal.bind(this))
            .registerSignalHandler(messages_1.SIGNAL_TYPE.waveformAudioVolumeChange, this._onVolumeChangeSignal.bind(this))
            .registerSignalHandler(messages_1.SIGNAL_TYPE.audioStop, this._onStopSignal.bind(this));
        return this;
    }
    start(config) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const parameters = yield this._rpc.rpc(messages_1.RPC_TYPE.getWaveformAudioParameters(this._index));
            this._toneGenerator.setConfig(config);
            this.setVolume(parameters.volume);
        });
    }
    setVolume(value) {
        this._volume = value;
        return this;
    }
    getVolume() {
        return this._volume;
    }
    getBuffer(key) {
        return this._toneGenerator.getBuffer(key);
    }
    _onVolumeChangeSignal(message) {
        if (message.index === this._index) {
            this._volume = message.value;
            this.volumeChanged.dispatch(this._volume);
        }
    }
    _onBufferChangeSignal(message) {
        if (message.index === this._index) {
            this.bufferChanged.dispatch(message.key);
        }
    }
    _onStopSignal(index) {
        if (index === this._index) {
            this.stop.dispatch(undefined);
        }
    }
}
exports.default = WaveformAudioProxy;
//# sourceMappingURL=WaveformAudioProxy.js.map